# Copyright 2015 OpenStack Foundation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from alembic import op
import sqlalchemy as sa

import murano.db.migration.helpers as helpers
from murano.db.sqla import types as st

"""fix_unique_constraints

Revision ID: 008
Revises: 007
Create Date: 2015-04-08 14:01:06.458512

"""

# revision identifiers, used by Alembic.
revision = '008'
down_revision = '007'

MYSQL_ENGINE = 'InnoDB'
MYSQL_CHARSET = 'utf8'


def upgrade():
    engine = op.get_bind()
    if engine.dialect.dialect_description.startswith('mysql'):
        engine.execute('SET FOREIGN_KEY_CHECKS=0')
    if engine.dialect.dialect_description == 'postgresql+psycopg2':
        op.drop_constraint('package_to_tag_package_id_fkey',
                           'package_to_tag', 'foreignkey')
        op.drop_constraint('package_to_tag_tag_id_fkey',
                           'package_to_tag', 'foreignkey')
        op.drop_constraint('package_to_category_package_id_fkey',
                           'package_to_category', 'foreignkey')
        op.drop_constraint('class_definition_package_id_fkey',
                           'class_definition', 'foreignkey')
    helpers.transform_table(
        'package', {}, {},
        sa.Column('created', sa.DateTime(), nullable=False),
        sa.Column('updated', sa.DateTime(), nullable=False),
        sa.Column('id', sa.String(length=36), nullable=False),
        sa.Column('archive', st.LargeBinary(), nullable=True),
        sa.Column('fully_qualified_name',
                  sa.String(length=128),
                  nullable=False),
        sa.Column('type', sa.String(length=20), nullable=False),
        sa.Column('author', sa.String(length=80), nullable=True),
        sa.Column('name', sa.String(length=80), nullable=False),
        sa.Column('enabled', sa.Boolean(), nullable=True),
        sa.Column('description', sa.Text(), nullable=False),
        sa.Column('is_public', sa.Boolean(), nullable=True),
        sa.Column('logo', st.LargeBinary(), nullable=True),
        sa.Column('owner_id', sa.String(length=36), nullable=False),
        sa.Column('ui_definition', sa.Text(), nullable=True),
        sa.Column('supplier_logo', sa.types.LargeBinary),
        sa.Column('supplier', sa.types.Text()),
        sa.PrimaryKeyConstraint('id'),
        mysql_engine=MYSQL_ENGINE,
        mysql_charset=MYSQL_CHARSET
    )

    helpers.transform_table(
        'class_definition', {}, {},
        sa.Column('created', sa.DateTime(), nullable=False),
        sa.Column('updated', sa.DateTime(), nullable=False),
        sa.Column('id', sa.String(length=36), nullable=False),
        sa.Column('name',
                  sa.String(length=128),
                  nullable=False),
        sa.Column('package_id', sa.String(length=36), nullable=True),
        sa.ForeignKeyConstraint(['package_id'], ['package.id'], ),
        sa.PrimaryKeyConstraint('id'),
        mysql_engine=MYSQL_ENGINE,
        mysql_charset=MYSQL_CHARSET
    )

    if engine.dialect.dialect_description.startswith('mysql'):
        engine.execute('SET FOREIGN_KEY_CHECKS=1')

    if engine.dialect.dialect_description == 'postgresql+psycopg2':
        op.create_foreign_key('package_to_tag_package_id_fkey',
                              'package_to_tag', 'package',
                              ['package_id'], ['id'])

        op.create_foreign_key('package_to_tag_tag_id_fkey',
                              'package_to_tag', 'tag',
                              ['tag_id'], ['id'])

        op.create_foreign_key('package_to_category_package_id_fkey',
                              'package_to_category', 'package',
                              ['package_id'], ['id'])

        op.create_foreign_key('class_definition_package_id_fkey',
                              'class_definition', 'package',
                              ['package_id'], ['id'])

    op.create_index('ix_package_fqn_and_owner', table_name='package',
                    columns=['fully_qualified_name', 'owner_id'], unique=True)
    op.create_index('ix_class_definition_name',
                    'class_definition',
                    ['name'])


def downgrade():
    op.drop_index('ix_package_fqn_and_owner', table_name='package')
    op.drop_index('ix_class_definition_name', table_name='class_definition')

    engine = op.get_bind()
    if engine.dialect.dialect_description.startswith('mysql'):
        engine.execute('SET FOREIGN_KEY_CHECKS=0')
    if engine.dialect.dialect_description == 'postgresql+psycopg2':
        op.drop_constraint('package_to_tag_package_id_fkey',
                           'package_to_tag', 'foreignkey')
        op.drop_constraint('package_to_tag_tag_id_fkey',
                           'package_to_tag', 'foreignkey')
        op.drop_constraint('package_to_category_package_id_fkey',
                           'package_to_category', 'foreignkey')
        op.drop_constraint('class_definition_package_id_fkey',
                           'class_definition', 'foreignkey')

    helpers.transform_table(
        'class_definition', {}, {},
        sa.Column('created', sa.DateTime(), nullable=False),
        sa.Column('updated', sa.DateTime(), nullable=False),
        sa.Column('id', sa.String(length=36), nullable=False),
        sa.Column('name',
                  sa.String(length=128),
                  nullable=False,
                  unique=True),
        sa.Column('package_id', sa.String(length=36), nullable=True),
        sa.PrimaryKeyConstraint('id'),
        mysql_engine=MYSQL_ENGINE,
        mysql_charset=MYSQL_CHARSET
    )

    helpers.transform_table(
        'package', {}, {},
        sa.Column('created', sa.DateTime(), nullable=False),
        sa.Column('updated', sa.DateTime(), nullable=False),
        sa.Column('id', sa.String(length=36), nullable=False),
        sa.Column('archive', st.LargeBinary(), nullable=True),
        sa.Column('fully_qualified_name',
                  sa.String(length=128),
                  nullable=False,
                  unique=True),
        sa.Column('type', sa.String(length=20), nullable=False),
        sa.Column('author', sa.String(length=80), nullable=True),
        sa.Column('name', sa.String(length=80), nullable=False),
        sa.Column('enabled', sa.Boolean(), nullable=True),
        sa.Column('description', sa.Text(), nullable=False),
        sa.Column('is_public', sa.Boolean(), nullable=True),
        sa.Column('logo', st.LargeBinary(), nullable=True),
        sa.Column('owner_id', sa.String(length=36), nullable=False),
        sa.Column('ui_definition', sa.Text(), nullable=True),
        sa.Column('supplier_logo', sa.types.LargeBinary),
        sa.Column('supplier', sa.types.Text()),
        sa.PrimaryKeyConstraint('id'),
        mysql_engine=MYSQL_ENGINE,
        mysql_charset=MYSQL_CHARSET
    )

    if engine.dialect.dialect_description.startswith('mysql'):
        engine.execute('SET FOREIGN_KEY_CHECKS=1')

    if engine.dialect.dialect_description == 'postgresql+psycopg2':
        op.create_foreign_key('package_to_tag_package_id_fkey',
                              'package_to_tag', 'package',
                              ['package_id'], ['id'])

        op.create_foreign_key('package_to_tag_tag_id_fkey',
                              'package_to_tag', 'tag',
                              ['tag_id'], ['id'])

        op.create_foreign_key('package_to_category_package_id_fkey',
                              'package_to_category', 'package',
                              ['package_id'], ['id'])

        op.create_foreign_key('class_definition_package_id_fkey',
                              'class_definition', 'package',
                              ['package_id'], ['id'])

    op.create_index('ix_class_definition_name',
                    'class_definition',
                    ['name'])



#    Copyright (c) 2016 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging
from oslo_messaging.rpc import client
from webob import exc

from murano.common.i18n import _
from murano.common import policy
from murano.common import wsgi
from murano.services import static_actions


LOG = logging.getLogger(__name__)


class Controller(object):

    def execute(self, request, body):
        policy.check("execute_action", request.context, {})

        class_name = body.get('className')
        method_name = body.get('methodName')
        if not class_name or not method_name:
            msg = _('Class name and method name must be specified for '
                    'static action')
            LOG.error(msg)
            raise exc.HTTPBadRequest(msg)

        args = body.get('parameters')
        pkg_name = body.get('packageName')
        class_version = body.get('classVersion', '=0')

        LOG.debug('StaticAction:Execute <MethodName: {0}, '
                  'ClassName: {1}>'.format(method_name, class_name))

        credentials = {
            'token': request.context.auth_token,
            'project_id': request.context.tenant,
            'user_id': request.context.user
        }

        try:
            return static_actions.StaticActionServices.execute(
                method_name, class_name, pkg_name, class_version, args,
                credentials)
        except client.RemoteError as e:
            LOG.error('Exception during call of the method {method_name}: '
                      '{exc}'.format(method_name=method_name, exc=str(e)))
            if e.exc_type in (
                    'NoClassFound', 'NoMethodFound', 'NoPackageFound',
                    'NoPackageForClassFound', 'MethodNotExposed',
                    'NoMatchingMethodException'):
                raise exc.HTTPNotFound(e.value)
            elif e.exc_type == 'ContractViolationException':
                raise exc.HTTPBadRequest(e.value)
            raise exc.HTTPServiceUnavailable(e.value)
        except ValueError as e:
            LOG.error('Exception during call of the method {method_name}: '
                      '{exc}'.format(method_name=method_name,
                                     exc=str(e)))
            raise exc.HTTPBadRequest(str(e))


def create_resource():
    return wsgi.Resource(Controller())



# Copyright 2016 AT&T Corp
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from oslo_service import service

from murano.common import engine
from murano.dsl import constants
from murano.dsl import helpers
from murano.dsl import murano_package
from murano.engine import mock_context_manager
from murano.engine import package_loader
from murano.tests.unit import base


class TestEngineService(base.MuranoTestCase):
    def setUp(self):
        super(TestEngineService, self).setUp()
        self.engine = engine.EngineService()
        self.addCleanup(mock.patch.stopall)

    @mock.patch.object(service.Service, 'reset')
    @mock.patch.object(service.Service, 'stop')
    @mock.patch.object(service.Service, 'start')
    @mock.patch('murano.common.rpc.get_server')
    def test_start_stop_reset(self, mock_get_server, mock_start,
                              mock_stop, mock_reset):
        self.engine.start()
        self.assertTrue(mock_get_server.called)
        self.assertTrue(mock_start.called)
        self.engine.stop()
        self.assertTrue(mock_stop.called)
        self.engine.reset()
        self.assertTrue(mock_reset.called)

    @mock.patch.object(service.Service, 'stop')
    @mock.patch.object(service.Service, 'start')
    @mock.patch('murano.common.rpc.get_server')
    def test_stop_graceful(self, mock_get_server, mock_start, mock_stop):
        self.engine.start()
        self.assertTrue(mock_get_server.called)
        self.assertTrue(mock_start.called)
        self.engine.stop(graceful=True)
        self.assertTrue(mock_stop.called)


class TestTaskExecutor(base.MuranoTestCase):
    def setUp(self):
        super(TestTaskExecutor, self).setUp()
        self.task = {
            'action': {
                'args': None,
                'object_id': 'my_obj_id',
                'method': 'my_method'
            },
            'model': {
                'SystemData': {
                    'Packages': 'my_packages'
                },
                'project_id': 'my_tenant_id',
                'user_id': 'my_user_id'
            },
            'token': 'my_token',
            'project_id': 'my_tenant_id',
            'user_id': 'my_user_id',
            'id': 'my_env_id'
        }
        self.task_executor = engine.TaskExecutor(self.task)
        self.task_executor._model = self.task['model']
        self.task_executor._session.token = self.task['token']
        self.task_executor._session.project_id = self.task['project_id']
        self.task_executor._session.user_id = self.task['user_id']
        self.task_executor._session.environment_owner_project_id_ = \
            self.task['model']['project_id']
        self.task_executor._session.environment_owner_user_id = \
            self.task['model']['user_id']
        (self.task_executor._session
            .system_attributes) = (self.task_executor._model.
                                   get('SystemData', {}))
        self.addCleanup(mock.patch.stopall)

    def test_properties(self):
        self.assertEqual(self.task['action'], self.task_executor.action)
        self.assertEqual(self.task_executor._session,
                         self.task_executor.session)
        self.assertEqual(self.task['model'], self.task_executor.model)

    @mock.patch('murano.common.engine.auth_utils.delete_trust')
    @mock.patch('murano.common.engine.auth_utils.create_trust')
    @mock.patch('murano.common.engine.package_loader.'
                'CombinedPackageLoader.import_fixation_table')
    @mock.patch('murano.common.engine.TaskExecutor._execute')
    def test_execute(self, mock_execute, mock_loader,
                     mock_create, mock_delete):
        mock_loader.return_value = {'SystemData': 'my_sys_data'}
        mock_create.return_value = 'trust_id'
        expected = {
            'action': {
                'result': '?',
                'isException': False
            }
        }
        mock_execute.return_value = expected
        result = self.task_executor.execute()

        self.assertEqual(expected, result)
        self.assertTrue(mock_execute.called)
        self.assertTrue(mock_loader.called)
        self.assertTrue(mock_create.called)
        self.assertTrue(mock_delete.called)

    def test_private_execute(self):
        mock_loader = mock.Mock()
        result = self.task_executor._execute(mock_loader)
        expected = {
            'action': {
                'result': None,
                'isException': False
            }
        }
        self.assertEqual(expected, result)

    @mock.patch('murano.common.engine.auth_utils.delete_trust')
    @mock.patch('murano.common.engine.auth_utils.create_trust')
    def test_trust(self, mock_create, mock_delete):
        mock_create.return_value = 'trust_id'
        self.task_executor._create_trust()
        self.assertEqual('trust_id', self.task_executor._session.trust_id)
        self.assertTrue(mock_create.called)

        self.task_executor._delete_trust()
        self.assertIsNone(self.task_executor._session.trust_id)
        self.assertTrue(mock_delete.called)


class TestStaticActionExecutor(base.MuranoTestCase):

    def setUp(self):
        super(TestStaticActionExecutor, self).setUp()

        self.action = {
            'method': 'TestAction',
            'args': {'name': 'foo'},
            'class_name': 'TestClass',
            'pkg_name': 'TestPackage',
            'class_version': '=0'
        }
        self.task = {
            'action': self.action,
            'token': 'test_token',
            'project_id': 'test_tenant',
            'user_id': 'test_user',
            'id': 'test_task_id'
        }
        self.task_executor = engine.StaticActionExecutor(self.task)
        self.assertIsInstance(self.task_executor._reporter,
                              engine.status_reporter.StatusReporter)
        self.assertEqual(self.task['id'],
                         self.task_executor._reporter._environment_id)

    def test_action_property(self):
        self.assertEqual(self.action, self.task_executor.action)

    def test_session_property(self):
        self.assertIsInstance(self.task_executor.session,
                              engine.execution_session.ExecutionSession)
        self.assertEqual('test_token', self.task_executor.session.token)
        self.assertEqual('test_tenant', self.task_executor.session.project_id)
        self.assertIsInstance(self.task_executor._model_policy_enforcer,
                              engine.enforcer.ModelPolicyEnforcer)
        self.assertEqual(
            self.task_executor.session,
            self.task_executor._model_policy_enforcer._execution_session)

    @mock.patch.object(engine, 'serializer')
    @mock.patch.object(engine.dsl_executor.MuranoDslExecutor, 'package_loader')
    def test_execute(self, mock_package_loader, mock_serializer):
        mock_class = mock.Mock()
        mock_package = mock.Mock(spec=murano_package.MuranoPackage)
        mock_package.find_class.return_value = mock_class
        mock_package_loader.load_package.return_value = mock_package
        version_spec = helpers.parse_version_spec(self.action['class_version'])

        self.task_executor.execute()
        mock_package_loader.load_package.assert_called_once_with(
            'TestPackage', version_spec)
        mock_package.find_class.assert_called_once_with(
            self.action['class_name'], search_requirements=False)
        mock_class.invoke.assert_called_once_with('TestAction', None, (),
                                                  {'name': 'foo'})
        self.assertTrue(mock_serializer.serialize.called)

    @mock.patch.object(engine, 'serializer')
    @mock.patch.object(engine.dsl_executor.MuranoDslExecutor, 'package_loader')
    def test_execute_without_package_name(self, mock_package_loader,
                                          mock_serializer):
        mock_class = mock.Mock()
        mock_package = mock.Mock(spec=murano_package.MuranoPackage)
        mock_package.find_class.return_value = mock_class
        mock_package_loader.load_class_package.return_value = mock_package
        version_spec = helpers.parse_version_spec(self.action['class_version'])
        self.task_executor.action['pkg_name'] = None

        self.task_executor.execute()
        mock_package_loader.load_class_package.assert_called_once_with(
            'TestClass', version_spec)
        mock_package.find_class.assert_called_once_with(
            self.action['class_name'], search_requirements=False)
        mock_class.invoke.assert_called_once_with('TestAction', None, (),
                                                  {'name': 'foo'})
        self.assertTrue(mock_serializer.serialize.called)


class TestSchemaEndpoint(base.MuranoTestCase):

    def setUp(self):
        super(TestSchemaEndpoint, self).setUp()
        context_manager = mock_context_manager.MockContextManager()
        self.context = context_manager.create_root_context(
            constants.RUNTIME_VERSION_1_5)

    @mock.patch('murano.common.engine.schema_generator')
    @mock.patch('murano.common.engine.package_loader')
    def test_generate_schema(self, mock_package_loader,
                             mock_schema_generator):
        mock_pkg_loader = mock.Mock(
            spec=package_loader.CombinedPackageLoader)
        mock_package_loader.CombinedPackageLoader().__enter__.return_value =\
            mock_pkg_loader
        mock_schema_generator.generate_schema.return_value = 'test_schema'

        arg1, arg2, arg3 = mock.Mock(), mock.Mock(), mock.Mock()
        test_args = (arg1, arg2, arg3)
        test_kwargs = {'foo': 'bar', 'class_name': 'test_class_name'}

        result = engine.SchemaEndpoint.generate_schema(
            self.context, *test_args, **test_kwargs)

        self.assertEqual('test_schema', result)
        mock_schema_generator.generate_schema.assert_called_once_with(
            mock_pkg_loader, mock.ANY, *test_args, **test_kwargs)


class TestTaskProcessingEndpoint(base.MuranoTestCase):

    def setUp(self):
        super(TestTaskProcessingEndpoint, self).setUp()

        self.action = {
            'method': 'TestAction',
            'args': {'name': 'foo'},
            'class_name': 'TestClass',
            'pkg_name': 'TestPackage',
            'class_version': '=0',
            'object_id': 'test_object_id'
        }
        self.task = {
            'action': self.action,
            'model': {
                'SystemData': {'TrustId': 'test_trust_id'},
                'project_id': 'test_tenant',
                'user_id': 'test_user'
            },
            'token': 'test_token',
            'project_id': 'test_tenant',
            'user_id': 'test_user',
            'id': 'test_task_id'
        }
        context_manager = mock_context_manager.MockContextManager()
        self.context = context_manager.create_root_context(
            constants.RUNTIME_VERSION_1_5)

    @mock.patch.object(engine.TaskExecutor, '_delete_trust')
    @mock.patch.object(engine, 'rpc')
    @mock.patch.object(engine.dsl_executor.MuranoDslExecutor, 'finalize')
    @mock.patch.object(engine, 'LOG')
    def test_handle_task(self, mock_log, mock_finalize, mock_rpc,
                         mock_delete_trust):
        mock_finalize.return_value = self.task['model']

        handle_task = engine.TaskProcessingEndpoint.handle_task
        handle_task(self.context, self.task)

        mock_delete_trust.assert_called_once_with()
        mock_rpc.api().process_result.assert_called_once_with(
            mock.ANY, 'test_task_id')
        self.assertEqual(2, mock_log.info.call_count)
        self.assertIn('Starting processing task:',
                      str(mock_log.info.mock_calls[0]))
        self.assertIn('Finished processing task:',
                      str(mock_log.info.mock_calls[1]))


class TestStaticActionEndpoint(base.MuranoTestCase):

    def setUp(self):
        super(TestStaticActionEndpoint, self).setUp()

        self.action = {
            'method': 'TestAction',
            'args': {'name': 'foo'},
            'class_name': 'TestClass',
            'pkg_name': 'TestPackage',
            'class_version': '=0',
            'object_id': 'test_object_id'
        }
        self.task = {
            'action': self.action,
            'model': {'SystemData': {'TrustId': 'test_trust_id'}},
            'token': 'test_token',
            'project_id': 'test_tenant',
            'user_id': 'test_user',
            'id': 'test_task_id'
        }
        context_manager = mock_context_manager.MockContextManager()
        self.context = context_manager.create_root_context(
            constants.RUNTIME_VERSION_1_5)

    @mock.patch('murano.dsl.serializer.serialize')
    @mock.patch('murano.common.engine.package_loader')
    @mock.patch.object(engine, 'LOG')
    def test_call_static_action(self, mock_log, mock_package_loader,
                                mock_serialize):
        mock_pkg_loader = mock.Mock(
            spec=package_loader.CombinedPackageLoader)
        mock_package_loader.CombinedPackageLoader().__enter__.return_value =\
            mock_pkg_loader

        expected = {
            'Objects': ['foo'],
            'ObjectsCopy': ['bar'],
            'Attributes': ['baz']
        }
        mock_serialize.return_value = expected

        call_static_action = engine.StaticActionEndpoint.call_static_action
        result = call_static_action(self.context, self.task)

        self.assertEqual(expected, result)
        self.assertEqual(2, mock_log.info.call_count)
        self.assertIn('Starting execution of static action:',
                      str(mock_log.info.mock_calls[0]))
        self.assertIn('Finished execution of static action:',
                      str(mock_log.info.mock_calls[1]))



#    Copyright (c) 2014 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from murano.dsl import constants
from murano.dsl import dsl
from murano.dsl import dsl_types
from murano.dsl import exceptions
from murano.dsl import helpers
from murano.dsl.principal_objects import garbage_collector
from murano.dsl import yaql_integration


class MuranoObject(dsl_types.MuranoObject):
    def __init__(self, murano_class, owner, object_id=None, name=None,
                 known_classes=None, this=None, metadata=None):
        self._initialized = False
        self._destroyed = False
        if known_classes is None:
            known_classes = {}
        if this is None:
            self._owner = owner
        self._object_id = object_id or helpers.generate_id()
        self._type = murano_class
        self._properties = {}
        self._parents = {}
        self._this = this
        self._name = name
        self._extension = None
        self._executor = helpers.weak_ref(helpers.get_executor())
        self._config = murano_class.package.get_class_config(
            murano_class.name)
        self._metadata = metadata
        if not isinstance(self._config, dict):
            self._config = {}
        known_classes[murano_class.name] = self
        for parent_class in murano_class.parents:
            name = parent_class.name
            if name not in known_classes:
                obj = MuranoObject(
                    parent_class, owner,
                    object_id=self._object_id,
                    known_classes=known_classes, this=self.real_this)

                self._parents[name] = known_classes[name] = obj
            else:
                self._parents[name] = known_classes[name]
        self._destruction_dependencies = []

    @property
    def extension(self):
        return self._extension

    @property
    def name(self):
        return self.real_this._name

    @property
    def metadata(self):
        return self.real_this._metadata

    @extension.setter
    def extension(self, value):
        self._extension = value

    def initialize(self, context, params, used_names=None):
        context = context.create_child_context()
        context[constants.CTX_ALLOW_PROPERTY_WRITES] = True
        object_store = helpers.get_object_store()
        for property_name in self.type.properties:
            spec = self.type.properties[property_name]
            if spec.usage == dsl_types.PropertyUsages.Config:
                if property_name in self._config:
                    property_value = self._config[property_name]
                else:
                    property_value = dsl.NO_VALUE
                self.set_property(property_name, property_value,
                                  dry_run=self._initialized)

        init = self.type.methods.get('.init')
        used_names = used_names or set()
        names = set(self.type.properties)
        if init:
            names.update(init.arguments_scheme.keys())
        last_errors = len(names)
        init_args = {}
        while True:
            errors = 0
            for property_name in names:
                if init and property_name in init.arguments_scheme:
                    spec = init.arguments_scheme[property_name]
                    is_init_arg = True
                else:
                    spec = self.type.properties[property_name]
                    is_init_arg = False

                if property_name in used_names:
                    continue
                if spec.usage in (dsl_types.PropertyUsages.Config,
                                  dsl_types.PropertyUsages.Static):
                    used_names.add(property_name)
                    continue
                if spec.usage == dsl_types.PropertyUsages.Runtime:
                    if not spec.has_default:
                        used_names.add(property_name)
                        continue
                    property_value = dsl.NO_VALUE
                else:
                    property_value = params.get(property_name, dsl.NO_VALUE)
                try:
                    if is_init_arg:
                        init_args[property_name] = property_value
                    else:
                        self.set_property(
                            property_name, property_value, context,
                            dry_run=self._initialized)
                    used_names.add(property_name)
                except exceptions.UninitializedPropertyAccessError:
                    errors += 1
                except exceptions.ContractViolationException:
                    if spec.usage != dsl_types.PropertyUsages.Runtime:
                        raise
            if not errors:
                break
            if errors >= last_errors:
                raise exceptions.CircularExpressionDependenciesError()
            last_errors = errors

        if (not object_store.initializing and
                self._extension is None and
                not self._initialized and
                not self._destroyed and
                not helpers.is_objects_dry_run_mode()):
            method = self.type.methods.get('__init__')
            if method:
                filtered_params = yaql_integration.filter_parameters(
                    method.body, **params)
                yield lambda: method.invoke(
                    self, filtered_params[0], filtered_params[1], context)

        for parent in self._parents.values():
            for t in parent.initialize(context, params, used_names):
                yield t

        def run_init():
            if init:
                context[constants.CTX_ARGUMENT_OWNER] = self.real_this
                init.invoke(self.real_this, (), init_args,
                            context.create_child_context())
            self._initialized = True

        if (not object_store.initializing and
                not helpers.is_objects_dry_run_mode() and
                not self._initialized and
                not self._destroyed):
            yield run_init

    @property
    def object_id(self):
        return self._object_id

    @property
    def type(self):
        return self._type

    @property
    def owner(self):
        if self._this is None:
            return self._owner
        else:
            return self.real_this.owner

    @property
    def real_this(self):
        return self._this or self

    @property
    def executor(self):
        return self._executor()

    @property
    def initialized(self):
        return self._initialized

    @property
    def destruction_dependencies(self):
        return self._destruction_dependencies

    def load_dependencies(self, dependencies):
        self._destruction_dependencies = []
        if not dependencies:
            return
        destruction_dependencies = dependencies.get('onDestruction', [])
        object_store = helpers.get_object_store()
        for record in destruction_dependencies:
            subscriber_id = record['subscriber']
            subscriber = object_store.get(subscriber_id)
            if not subscriber:
                continue
            garbage_collector.GarbageCollector.subscribe_destruction(
                self, subscriber, record.get('handler'))

    def get_property(self, name, context=None):
        start_type, derived = self.type, False
        caller_class = None if not context else helpers.get_type(context)
        if caller_class is not None and caller_class.is_compatible(self):
            start_type, derived = caller_class, True

        declared_properties = start_type.find_properties(
            lambda p: p.name == name)
        if len(declared_properties) > 0:
            spec = self.real_this.type.find_single_property(name)
            if spec.usage == dsl_types.PropertyUsages.Static:
                return self.executor.get_static_property(
                    spec.declaring_type, name, context)
            else:
                return self.real_this._get_property_value(name)
        elif derived:
            return self.cast(caller_class)._get_property_value(name)
        else:
            raise exceptions.PropertyReadError(name, start_type)

    def _get_property_value(self, name):
        try:
            return self._properties[name]
        except KeyError:
            raise exceptions.UninitializedPropertyAccessError(
                name, self.type)

    def set_property(self, name, value, context=None, dry_run=False):
        start_type, derived = self.real_this.type, False
        caller_class = None if not context else helpers.get_type(context)
        if caller_class is not None and caller_class.is_compatible(self):
            start_type, derived = caller_class, True
        if context is None:
            context = self.executor.create_object_context(self)
        declared_properties = start_type.find_properties(
            lambda p: p.name == name)
        if len(declared_properties) > 0:
            ultimate_spec = self.real_this.type.find_single_property(name)
            property_list = list(self._list_properties(name))
            for spec in property_list:
                if (caller_class is not None and not
                        helpers.are_property_modifications_allowed(context) and
                        (spec.usage not in dsl_types.PropertyUsages.Writable or
                            not derived)):
                    raise exceptions.NoWriteAccessError(name)

                if spec.usage == dsl_types.PropertyUsages.Static:
                    default = None
                else:
                    default = self._config.get(name, spec.default)

                if spec is ultimate_spec:
                    value = spec.transform(
                        value, self.real_this,
                        self.real_this, context, default=default,
                        finalize=len(property_list) == 1)
                else:
                    spec.validate(value, self.real_this, context, default)
            if len(property_list) > 1:
                value = ultimate_spec.finalize(value, self.real_this, context)
            if ultimate_spec.usage == dsl_types.PropertyUsages.Static:
                self.executor.set_static_property(
                    ultimate_spec.declaring_type, name, value, context,
                    dry_run=dry_run)
            elif not dry_run:
                self.real_this._properties[name] = value
        elif derived:
            if not dry_run:
                obj = self.cast(caller_class)
                obj._properties[name] = value
        else:
            raise exceptions.PropertyWriteError(name, start_type)

    def cast(self, cls):
        for p in helpers.traverse(self, lambda t: t._parents.values()):
            if p.type == cls:
                return p
        raise TypeError('Cannot cast {0} to {1}'.format(self.type, cls))

    def _list_properties(self, name):
        for p in helpers.traverse(
                self.real_this, lambda t: t._parents.values()):
            if name in p.type.properties:
                yield p.type.properties[name]

    def __repr__(self):
        return '<{0}/{1} {2} ({3})>'.format(
            self.type.name, self.type.version, self.object_id, id(self))

    def to_dictionary(self, include_hidden=False,
                      serialization_type=dsl_types.DumpTypes.Serializable,
                      allow_refs=False, with_destruction_dependencies=False):
        context = helpers.get_context()
        result = {}
        for parent in self._parents.values():
            result.update(parent.to_dictionary(
                include_hidden, dsl_types.DumpTypes.Serializable,
                allow_refs))
        skip_usages = (dsl_types.PropertyUsages.Runtime,
                       dsl_types.PropertyUsages.Config)
        for property_name in self.type.properties:
            if property_name in self.real_this._properties:
                spec = self.type.properties[property_name]
                if spec.usage not in skip_usages or include_hidden:
                    prop_value = self.real_this._properties[property_name]
                    if isinstance(prop_value, MuranoObject) and allow_refs:
                        meta = [m for m in spec.get_meta(context)
                                if m.type.name == ('io.murano.metadata.'
                                                   'engine.Serialize')]
                        if meta and meta[0].get_property(
                                'as', context) == 'reference':
                            prop_value = prop_value.object_id
                    result[property_name] = prop_value
        if serialization_type == dsl_types.DumpTypes.Inline:
            result.pop('?')
            result = {
                self.type: result,
                'id': self.object_id,
                'name': self.name,
                'metadata': self.metadata
            }
            header = result
        else:
            if serialization_type == dsl_types.DumpTypes.Mixed:
                result.update({'?': {
                    'type': self.type,
                    'id': self.object_id,
                    'name': self.name,
                    'metadata': self.metadata
                }})
            else:
                result.update({'?': {
                    'type': helpers.format_type_string(self.type),
                    'id': self.object_id,
                    'name': self.name,
                    'metadata': self.metadata
                }})
            header = result['?']
        if self.destroyed:
            header['destroyed'] = True
        if with_destruction_dependencies:
            dds = []
            for record in self.destruction_dependencies:
                subscriber = record['subscriber']()
                if not subscriber or self.executor.object_store.is_doomed(
                        subscriber):
                    continue
                dds.append({
                    'subscriber': subscriber.object_id,
                    'handler': record['handler']
                })
            if dds:
                header.setdefault('dependencies', {})['onDestruction'] = dds
        return result

    def mark_destroyed(self, clear_data=False):
        self._destroyed = True
        self._suppress__del__ = None
        if clear_data or not self.initialized:
            self._extension = None
            self._properties = None
            self._owner = None
            self._destruction_dependencies = None
            self._this = None
        for p in self._parents.values():
            p.mark_destroyed(clear_data)

    @property
    def destroyed(self):
        return self._destroyed


class RecyclableMuranoObject(MuranoObject):
    def __init__(self, *args, **kwargs):
        # Create self-reference to prevent __del__ from being called
        # automatically when there are no other objects referring to this one.
        # Without this reference __del__ will get called immediately after
        # reference counter will go to 0 and the object will put itself into
        # pending list creating another reference to itself and thus preventing
        # its child objects from being deleted. After the .destroy method
        # child objects will become eligible for destruction but will be
        # unable to use find() method since their owner will be destroyed
        # and collected at that point. With this reference gc.collect()
        # will collect the whole object graph at once and then we could
        # sort it and destroy in the correct order so that child objects
        # will be destroyed first.

        self._suppress__del__ = self
        super(RecyclableMuranoObject, self).__init__(*args, **kwargs)

    def __del__(self):
        # For Py2 the purpose of __del__ (in combination with _suppress__del__)
        # method is just to prevent object from being released automatically.
        # In Py3 the gc.collect list will be empty and __del__ will be called
        # for objects that were not destroyed yet.
        if self._this is None and self._initialized and not self._destroyed:
            self.executor.object_store.schedule_object_destruction(self)

    def mark_destroyed(self, clear_data=False):
        self.executor.attribute_store.forget_object(self)
        super(RecyclableMuranoObject, self).mark_destroyed(clear_data)



#    Copyright (c) 2014 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re

TYPE_NAME_RE = re.compile(r'^([a-zA-Z_]\w*:|:)?[a-zA-Z_]\w*(\.[a-zA-Z_]\w*)*$')
NS_RE = re.compile(r'^([a-zA-Z_]\w*(\.[a-zA-Z_]\w*)*)?$')
PREFIX_RE = re.compile(r'^([a-zA-Z_]\w*|=)$')


class NamespaceResolver(object):
    def __init__(self, namespaces):
        if namespaces is None:
            namespaces = {}
        for prefix, ns in namespaces.items():
            if ns is None:
                ns = ''
            if PREFIX_RE.match(prefix) is None:
                raise ValueError(
                    'Invalid namespace prefix "{0}"'.format(prefix))
            if NS_RE.match(ns) is None:
                raise ValueError('Invalid namespace "{0}"'.format(ns))
        self._namespaces = namespaces.copy()
        self._namespaces.setdefault('=', '')
        self._namespaces[''] = ''

    def resolve_name(self, name):
        if not self.is_typename(name, True):
            raise ValueError('Invalid type name "{0}"'.format(name))
        name = str(name)
        if ':' not in name:
            if '.' in name:
                parts = ['', name]
            else:
                parts = ['=', name]
        else:
            parts = name.split(':')
            if not parts[0]:
                parts[0] = '='

        if parts[0] not in self._namespaces:
            raise KeyError('Unknown namespace prefix ' + parts[0])

        ns = self._namespaces[parts[0]]
        if not ns:
            return parts[1]
        return '.'.join((ns, parts[1]))

    @staticmethod
    def is_typename(name, relaxed):
        if not name:
            return False
        name = str(name)
        if not relaxed and ':' not in name:
            return False
        return TYPE_NAME_RE.match(name) is not None



# Copyright 2016 OpenStack Foundation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Increase time resolution for status reports

Revision ID: 015
Create Date: 2016-06-17

"""

from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '015'
down_revision = '014'

MYSQL_ENGINE = 'InnoDB'
MYSQL_CHARSET = 'utf8'


def upgrade():
    op.add_column('environment', sa.Column('description_text', sa.Text(),
                  nullable=True))
    op.add_column('environment-template', sa.Column('description_text',
                                                    sa.Text(),
                  nullable=True))


def downgrade():
    with op.batch_alter_table("environment") as batch_op:
        batch_op.drop_column('description_text')
    with op.batch_alter_table("environment-template") as batch_op2:
        batch_op2.drop_column('description_text')



# Copyright (c) 2014 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import inspect
import os.path

import eventlet.debug

from murano.tests.unit import base
from murano.tests.unit.dsl.foundation import runner
from murano.tests.unit.dsl.foundation import test_package_loader


class DslTestCase(base.MuranoTestCase):
    def setUp(self):
        super(DslTestCase, self).setUp()
        directory = os.path.join(os.path.dirname(
            inspect.getfile(self.__class__)), 'meta')
        root_meta_directory = os.path.join(
            os.path.dirname(__file__), '../../../../../meta')
        sys_package_loader = test_package_loader.TestPackageLoader(
            os.path.join(root_meta_directory, 'io.murano/Classes'),
            'io.murano')
        self._package_loader = test_package_loader.TestPackageLoader(
            directory, 'tests', sys_package_loader)
        self._functions = {}
        self.register_function(
            lambda data: self._traces.append(data), 'trace')
        self._traces = []
        self._runners = []
        eventlet.debug.hub_exceptions(False)

    def new_runner(self, model):
        r = runner.Runner(model, self.package_loader, self._functions)
        self._runners.append(r)
        return r

    def tearDown(self):
        super(DslTestCase, self).tearDown()
        for r in self._runners:
            r.executor.finalize(r.root)

    @property
    def traces(self):
        return self._traces

    @traces.deleter
    def traces(self):
        self._traces = []

    @property
    def package_loader(self):
        return self._package_loader

    def register_function(self, func, name):
        self._functions[name] = func

    @staticmethod
    def find_attribute(model, obj_id, obj_type, name):
        for entry in model['Attributes']:
            if tuple(entry[:3]) == (obj_id, obj_type, name):
                return entry[3]



#    Copyright (c) 2013 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import collections

SessionState = collections.namedtuple('SessionState', [
    'OPENED', 'DEPLOYING', 'DEPLOYED', 'DEPLOY_FAILURE', 'DELETING',
    'DELETE_FAILURE'
])(
    OPENED='opened',
    DEPLOYING='deploying',
    DEPLOYED='deployed',
    DEPLOY_FAILURE='deploy failure',
    DELETING='deleting',
    DELETE_FAILURE='delete failure'
)

EnvironmentStatus = collections.namedtuple('EnvironmentStatus', [
    'READY', 'PENDING', 'DEPLOYING', 'DEPLOY_FAILURE', 'DELETING',
    'DELETE_FAILURE'
])(
    READY='ready',
    PENDING='pending',
    DEPLOYING='deploying',
    DEPLOY_FAILURE='deploy failure',
    DELETING='deleting',
    DELETE_FAILURE='delete failure'

)



#    Copyright (c) 2013 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time

from oslo_log import log as logging

from murano.api import v1
from murano.common import wsgi
from murano.db.services import stats

LOG = logging.getLogger(__name__)


class RequestStatisticsCollection(object):
    request_count = 0
    error_count = 0
    average_time = 0.0

    requests_per_tenant = {}
    errors_per_tenant = {}

    def add_api_request(self, tenant, ex_time):
        self.average_time = (self.average_time * self.request_count +
                             ex_time) / (self.request_count + 1)
        if tenant:
            tenant_count = self.requests_per_tenant.get(tenant, 0)
            tenant_count += 1
            self.requests_per_tenant[tenant] = tenant_count

    def add_api_error(self, tenant, ex_time):
        self.average_time = (self.average_time * self.request_count +
                             ex_time) / (self.request_count + 1)
        if tenant:
            tenant_count = self.errors_per_tenant.get(tenant, 0)
            tenant_count += 1
            self.errors_per_tenant[tenant] = tenant_count


def stats_count(api, method):
    def wrapper(func):
        def wrap(*args, **kwargs):
            try:
                ts = time.time()
                result = func(*args, **kwargs)
                te = time.time()
                tenant = args[1].context.tenant
                update_count(api, method, te - ts,
                             tenant)
                return result
            except Exception:
                te = time.time()
                tenant = args[1].context.tenant
                LOG.exception('API {api} method {method} raised an '
                              'exception'.format(api=api, method=method))
                update_error_count(api, method, te - te, tenant)
                raise
        return wrap

    return wrapper


def update_count(api, method, ex_time, tenant=None):
    LOG.debug("Updating count stats for {api}, {method} on object {object}"
              .format(api=api, method=method, object=v1.stats))
    v1.stats.add_api_request(tenant, ex_time)
    v1.stats.request_count += 1


def update_error_count(api, method, ex_time, tenant=None):
    LOG.debug("Updating count stats for {api}, {method} on object "
              "{object}".format(api=api, method=method, object=v1.stats))
    v1.stats.add_api_error(tenant, ex_time)
    v1.stats.error_count += 1
    v1.stats.request_count += 1


def init_stats():
    if not v1.stats:
        v1.stats = RequestStatisticsCollection()


class Controller(object):
    def get(self, request):
        model = stats.Statistics()
        entries = model.get_all()
        ent_list = []
        for entry in entries:
            ent_list.append(entry.to_dict())
        return ent_list


def create_resource():
    return wsgi.Resource(Controller())



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Change sizes of columns that hold keystone user ID to support domain users
which are 64 characters long.

Revision ID: 012
Revises: table session, table package

"""

from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '012'
down_revision = '011'

MYSQL_ENGINE = 'InnoDB'
MYSQL_CHARSET = 'utf8'


def upgrade():
    with op.batch_alter_table("session") as batch_op:
        batch_op.alter_column('user_id', type_=sa.String(64), nullable=False)
    with op.batch_alter_table("package") as batch_op2:
        batch_op2.alter_column('owner_id', type_=sa.String(64), nullable=False)
    # end Alembic commands #


def downgrade():
    with op.batch_alter_table("session") as batch_op:
        batch_op.alter_column('user_id', type_=sa.String(36), nullable=False)
    with op.batch_alter_table("package") as batch_op2:
        batch_op2.alter_column('owner_id', type_=sa.String(36), nullable=False)
    # end Alembic commands #



#    Copyright (c) 2016 AT&T
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from murano.db import models
from murano.engine.system import instance_reporter
from murano.tests.unit import base

LATEST_VERSION = 1


class TestInstanceReporter(base.MuranoTestCase):
    def setUp(self):
        super(TestInstanceReporter, self).setUp()

        self.environment = models.Environment(
            name='test_environment', tenant_id='test_tenant_id',
            version=LATEST_VERSION
        )

        self.addCleanup(mock.patch.stopall)

    @mock.patch("murano.db.models")
    def test_track_untrack_application(self, mock_models):
        instance = mock_models.Instance()
        self.i_r = instance_reporter.InstanceReportNotifier(self.environment)
        self.assertEqual(self.environment.id, self.i_r._environment_id)
        self.assertIsNone(self.i_r.track_application(instance))
        self.assertIsNone(self.i_r.untrack_application(instance))

    @mock.patch("murano.db.models")
    def test_track_untrack_cloud_instance(self, mock_models):
        instance = mock_models.Instance()
        self.i_r = instance_reporter.InstanceReportNotifier(self.environment)
        self.assertEqual(self.environment.id, self.i_r._environment_id)
        self.assertIsNone(self.i_r.track_cloud_instance(instance))
        self.assertIsNone(self.i_r.untrack_cloud_instance(instance))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from murano.dsl import context_manager
from murano.dsl import yaql_integration
from murano.tests.unit.dsl.foundation import test_case


class TestContextManager(test_case.DslTestCase):
    def setUp(self):
        super(TestContextManager, self).setUp()
        self.context_manager = context_manager.ContextManager()

    @mock.patch.object(yaql_integration, 'create_context', return_value='foo')
    def test_create_root_context(self, mock_create_context):
        val = self.context_manager.create_root_context('myrunver')

        self.assertEqual('foo', val)
        mock_create_context.assert_called_with('myrunver')

    def test_create_package_context(self):
        package = mock.MagicMock(context='mycontext')
        self.assertEqual('mycontext',
                         self.context_manager.create_package_context(package))

    def test_create_type_context(self):
        murano_type = mock.MagicMock(context='mycontext')
        self.assertEqual('mycontext',
                         self.context_manager.create_type_context(murano_type))

    def test_create_object_context(self):
        obj = 'obj'
        self.assertIsNone(self.context_manager.create_object_context(obj))



# Copyright 2016 AT&T Corp
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from murano.api.middleware import fault
from murano.common import wsgi
from murano.packages import exceptions
from murano.tests.unit import base

from oslo_serialization import jsonutils

from webob import exc


class FaultWrapperTest(base.MuranoTestCase):
    @mock.patch('traceback.format_exc')
    def test_error_500(self, mock_trace):
        mock_trace.return_value = "test trace"
        fault_wrapper = fault.FaultWrapper(None)
        result = fault_wrapper._error(exc.HTTPInternalServerError())
        self.assertEqual(result['code'], 500)
        self.assertEqual(result['explanation'],
                         'The server has either erred or is incapable'
                         ' of performing the requested operation.')

    @mock.patch('traceback.format_exc')
    def test_error_value_error(self, mock_trace):
        mock_trace.return_value = "test trace"
        fault_wrapper = fault.FaultWrapper(None)
        exception = exceptions.PackageClassLoadError("test")
        exception.message = "Unable to load class 'test' from package"
        result = fault_wrapper._error(exception)
        self.assertEqual(result['code'], 400)
        self.assertEqual(result['error']['message'],
                         "Unable to load class 'test' from package")

    @mock.patch('traceback.format_exc')
    def test_fault_wrapper(self, mock_trace):
        mock_trace.return_value = "test trace"
        fault_wrapper = fault.FaultWrapper(None)
        exception_disguise = fault.HTTPExceptionDisguise(
            exc.HTTPInternalServerError())
        result = fault_wrapper._error(exception_disguise)
        self.assertEqual(result['code'], 500)
        self.assertEqual(result['explanation'],
                         'The server has either erred or is incapable'
                         ' of performing the requested operation.')

    def test_process_request(self):
        fault_wrapper = fault.FaultWrapper(None)
        environ = {
            'SERVER_NAME': 'server.test',
            'SERVER_PORT': '8082',
            'SERVER_PROTOCOL': 'http',
            'SCRIPT_NAME': '/',
            'PATH_INFO': '/asdf/asdf/asdf/asdf',
            'wsgi.url_scheme': 'http',
            'QUERY_STRING': '',
            'CONTENT_TYPE': 'application/json',
            'REQUEST_METHOD': 'HEAD'
        }
        req = wsgi.Request(environ)
        req.get_response = mock.MagicMock(side_effect=exc.
                                          HTTPInternalServerError())
        self.assertRaises(exc.HTTPInternalServerError,
                          fault_wrapper.process_request, req)

    @mock.patch('traceback.format_exc')
    def test_fault_call(self, mock_trace):
        mock_trace.return_value = "test trace"
        fault_wrapper = fault.FaultWrapper(None)
        exception = exceptions.PackageClassLoadError("test")
        exception.message = "Unable to load class 'test' from package"
        test_fault = fault.Fault(fault_wrapper._error(exception))
        environ = {
            'SERVER_NAME': 'server.test',
            'SERVER_PORT': '8082',
            'SERVER_PROTOCOL': 'http',
            'SCRIPT_NAME': '/',
            'PATH_INFO': '/',
            'wsgi.url_scheme': 'http',
            'QUERY_STRING': '',
            'CONTENT_TYPE': 'application/json',
            'REQUEST_METHOD': 'HEAD'
        }
        req = wsgi.Request(environ)
        response = jsonutils.loads(test_fault(req).body)
        self.assertEqual(response['code'], 400)



# Copyright (c) 2016 AT&T
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import random
from unittest import mock

try:
    from mistralclient.api import client as mistralcli
except ImportError:
    mistralcli = None

from oslo_config import cfg

from murano.dsl import murano_method
from murano.dsl import murano_type

from murano.engine.system import workflowclient
from murano.tests.unit import base


CONF = cfg.CONF


def rand_name(name='murano'):
    """Generates random string.

    :param name: Basic name
    :return:
    """
    return name + str(random.randint(1, 0x7fffffff))


class TestMistralClient(base.MuranoTestCase):
    def setUp(self):
        super(TestMistralClient, self).setUp()
        self.mistral_client_mock = mock.Mock()
        self.mistral_client_mock.client = mock.MagicMock(
            spec=mistralcli.client)
        self._patch_client()

        self.mock_class = mock.MagicMock(spec=murano_type.MuranoClass)
        self.mock_method = mock.MagicMock(spec=murano_method.MuranoMethod)

        self._this = mock.MagicMock()
        self._this.owner = None

        self.addCleanup(mock.patch.stopall)

    def _patch_client(self):
        self.mock_client = mock.Mock(return_value=self.mistral_client_mock)
        self.client_patcher = mock.patch.object(workflowclient.MistralClient,
                                                '_client', self.mock_client)
        self.client_patcher.start()

        self.mock_create_client = mock.Mock(
            return_value=self.mistral_client_mock)
        self.create_client_patcher = mock.patch.object(
            workflowclient.MistralClient, '_create_client',
            self.mock_create_client)
        self.create_client_patcher.start()

    def _unpatch_client(self):
        self.client_patcher.stop()
        self.create_client_patcher.stop()

    def test_run_with_execution_success_state(self):
        test_output = '{"openstack": "foo", "__execution": "bar", "task":'\
                      ' "baz"}'
        mock_execution = mock.MagicMock(
            id='123', state='SUCCESS', output=test_output)
        self.mock_client.executions.create.return_value = mock_execution
        self.mock_client.executions.get.return_value = mock_execution

        run_name = rand_name('test')
        timeout = 1
        mc = workflowclient.MistralClient(self._this, 'regionOne')
        output = mc.run(run_name, timeout)

        for prop in ['openstack', '__execution', 'task']:
            self.assertFalse(hasattr(output, prop))

        self.assertEqual({}, output)

    def test_run_with_execution_error_state(self):
        mock_execution = mock.MagicMock(
            id='123', state='ERROR', output="{'test_attr': 'test_val'}")
        self.mock_client.executions.create.return_value = mock_execution
        self.mock_client.executions.get.return_value = mock_execution

        run_name = rand_name('test')
        timeout = 1
        mc = workflowclient.MistralClient(self._this, 'regionOne')

        expected_error_msg = 'Mistral execution completed with ERROR.'\
                             ' Execution id: {0}. Output: {1}'\
                             .format(mock_execution.id, mock_execution.output)
        with self.assertRaisesRegex(workflowclient.MistralError,
                                    expected_error_msg):
            mc.run(run_name, timeout)

    def test_run_except_timeout_error(self):
        mock_execution = mock.MagicMock(
            id='123', state='TEST_STATE', output="{'test_attr': 'test_val'}")
        self.mock_client.executions.create.return_value = mock_execution
        self.mock_client.executions.get.return_value = mock_execution

        run_name = rand_name('test')
        timeout = 1
        mc = workflowclient.MistralClient(self._this, 'regionOne')

        expected_error_msg = 'Mistral run timed out. Execution id: {0}.'\
                             .format(mock_execution.id)
        with self.assertRaisesRegex(workflowclient.MistralError,
                                    expected_error_msg):
            mc.run(run_name, timeout)

    def test_run_with_immediate_timeout(self):
        mock_execution = mock.MagicMock(
            id='123', state='ERROR', output="{'test_attr': 'test_val'}")
        self.mock_client.executions.create.return_value = mock_execution

        run_name = rand_name('test')
        timeout = 0
        mc = workflowclient.MistralClient(self._this, 'regionOne')
        self.assertEqual(mock_execution.id, mc.run(run_name, timeout))

    def test_upload(self):
        mc = workflowclient.MistralClient(self._this, 'regionOne')
        definition = rand_name('test')
        self.assertIsNone(mc.upload(definition))
        self.assertTrue(workflowclient.MistralClient.
                        _client.workflows.create.called)

    @mock.patch('murano.engine.system.workflowclient.auth_utils')
    def test_client_property(self, _):
        self._unpatch_client()

        test_mistral_settings = {
            'url': rand_name('test_mistral_url'),
            'project_id': rand_name('test_project_id'),
            'endpoint_type': rand_name('test_endpoint_type'),
            'auth_token': rand_name('test_auth_token'),
            'user_id': rand_name('test_user_id'),
            'insecure': rand_name('test_insecure'),
            'cacert': rand_name('test_ca_cert')
        }

        with mock.patch('murano.engine.system.workflowclient.CONF')\
                as mock_conf:
            mock_conf.mistral = mock.MagicMock(**test_mistral_settings)
            region_name = rand_name('test_region_name')
            mc = workflowclient.MistralClient(self._this, region_name)

            mistral_client = mc._client
            self.assertIsNotNone(mistral_client)



#    Copyright (c) 2015 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import routes

from murano.cfapi import cfapi
from murano.common import wsgi


class API(wsgi.Router):
    @classmethod
    def factory(cls, global_conf, **local_conf):
        return cls(routes.Mapper())

    def __init__(self, mapper):
        services_resource = cfapi.create_resource()
        mapper.connect('/v2/catalog',
                       controller=services_resource,
                       action='list',
                       conditions={'method': ['GET']})
        mapper.connect(('/v2/service_instances/{instance_id}'),
                       controller=services_resource,
                       action='provision',
                       conditions={'method': ['PUT']})
        mapper.connect(('/v2/service_instances/{instance_id}'),
                       controller=services_resource,
                       action='deprovision',
                       conditions={'method': ['DELETE']})

        mapper.connect(('/v2/service_instances/{instance_id}/service_bindings/'
                        '{app_id}'),
                       controller=services_resource,
                       action='bind',
                       conditions={'method': ['PUT']})

        mapper.connect(('/v2/service_instances/{instance_id}/service_bindings/'
                        '{app_id}'),
                       controller=services_resource,
                       action='unbind',
                       conditions={'method': ['DELETE']})

        mapper.connect(('/v2/service_instances/{instance_id}/last_operation'),
                       controller=services_resource,
                       action='get_last_operation',
                       conditions={'method': ['GET']})

        super(API, self).__init__(mapper)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc
import imghdr
import os
import re
import sys

import semantic_version

from murano.common.helpers import path
from murano.common.i18n import _
from murano.common import utils
from murano.packages import exceptions
from murano.packages import package


class PackageBase(package.Package):
    def __init__(self, format_name, runtime_version,
                 source_directory, manifest):
        super(PackageBase, self).__init__(
            format_name, runtime_version, source_directory)
        self._full_name = manifest.get('FullName')
        if not self._full_name:
            raise exceptions.PackageFormatError('FullName is not specified')
        self._check_full_name(self._full_name)
        self._version = semantic_version.Version.coerce(str(manifest.get(
            'Version', '0.0.0')))
        self._package_type = manifest.get('Type')
        if self._package_type not in package.PackageType.ALL:
            raise exceptions.PackageFormatError(
                'Invalid package Type {0}'.format(self._package_type))
        self._display_name = manifest.get('Name', self._full_name)
        self._description = manifest.get('Description')
        self._author = manifest.get('Author')
        self._supplier = manifest.get('Supplier') or {}
        self._logo = manifest.get('Logo')
        self._tags = manifest.get('Tags', [])

        self._logo_cache = None
        self._supplier_logo_cache = None
        self._source_directory = source_directory

    @abc.abstractproperty
    def requirements(self):
        raise NotImplementedError()

    @abc.abstractproperty
    def classes(self):
        raise NotImplementedError()

    @abc.abstractmethod
    def get_class(self, name):
        raise NotImplementedError()

    @abc.abstractproperty
    def ui(self):
        raise NotImplementedError()

    @property
    def full_name(self):
        return self._full_name

    @property
    def source_directory(self):
        return self._source_directory

    @property
    def version(self):
        return self._version

    @property
    def package_type(self):
        return self._package_type

    @property
    def display_name(self):
        return self._display_name

    @property
    def description(self):
        return self._description

    @property
    def author(self):
        return self._author

    @property
    def supplier(self):
        return self._supplier

    @property
    def tags(self):
        return list(self._tags)

    @property
    def logo(self):
        return self._load_image(self._logo, 'logo.png', 'logo')

    @property
    def meta(self):
        return None

    @property
    def supplier_logo(self):
        return self._load_image(
            self._supplier.get('Logo'), 'supplier_logo.png', 'supplier logo')

    def get_resource(self, name):
        resources_dir = path.secure_join(self._source_directory, 'Resources')
        if not os.path.exists(resources_dir):
            os.makedirs(resources_dir)
        return path.secure_join(resources_dir, name)

    def _load_image(self, file_name, default_name, what_image):
        full_path = path.secure_join(
            self._source_directory, file_name or default_name)
        if not os.path.isfile(full_path) and not file_name:
            return

        allowed_ftype = ('png', 'jpeg', 'gif')
        allowed_size = 500 * 1024
        try:

            if imghdr.what(full_path) not in allowed_ftype:
                msg = _('{0}: Unsupported Format. Only {1} allowed').format(
                    what_image, ', '.join(allowed_ftype))

                raise exceptions.PackageLoadError(msg)

            fsize = os.stat(full_path).st_size
            if fsize > allowed_size:
                msg = _('{0}: Uploaded image size {1} is too large. '
                        'Max allowed size is {2}').format(
                    what_image, fsize, allowed_size)
                raise exceptions.PackageLoadError(msg)

            with open(full_path, 'rb') as stream:
                return stream.read()

        except Exception as ex:
            trace = sys.exc_info()[2]
            utils.reraise(
                exceptions.PackageLoadError,
                exceptions.PackageLoadError(
                    'Unable to load {0}: {1}'.format(what_image, ex)),
                trace)

    @staticmethod
    def _check_full_name(full_name):
        error = exceptions.PackageFormatError('Invalid FullName ' + full_name)
        if re.match(r'^[\w\.]+$', full_name):
            if full_name.startswith('.') or full_name.endswith('.'):
                raise error
            if '..' in full_name:
                raise error
        else:
            raise error



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging
from yaql.language import specs
from yaql.language import yaqltypes

from murano.dsl import dsl

NAME_TEMPLATE = u'applications.{0}'

inject_format = specs.inject(
    '_Logger__yaql_format_function',
    yaqltypes.Delegate('format'))


@dsl.name('io.murano.system.Logger')
class Logger(object):
    """Logger object for MuranoPL.

    Instance of this object returned by 'logger' YAQL function
    and should not be instantiated directly
    """

    def __init__(self, logger_name):
        self._underlying_logger = logging.getLogger(
            NAME_TEMPLATE.format(logger_name))

    @specs.parameter('_Logger__message', yaqltypes.String())
    @inject_format
    def trace(__self, __yaql_format_function, __message, *args, **kwargs):
        __self._log(__self._underlying_logger.trace,
                    __yaql_format_function, __message, args, kwargs)

    @specs.parameter('_Logger__message', yaqltypes.String())
    @inject_format
    def debug(__self, __yaql_format_function, __message, *args, **kwargs):
        __self._log(__self._underlying_logger.debug,
                    __yaql_format_function, __message, args, kwargs)

    @specs.parameter('_Logger__message', yaqltypes.String())
    @inject_format
    def info(__self, __yaql_format_function, __message, *args, **kwargs):
        __self._log(__self._underlying_logger.info,
                    __yaql_format_function, __message, args, kwargs)

    @specs.parameter('_Logger__message', yaqltypes.String())
    @inject_format
    def warning(__self, __yaql_format_function, __message, *args, **kwargs):
        __self._log(__self._underlying_logger.warning,
                    __yaql_format_function, __message, args, kwargs)

    @specs.parameter('_Logger__message', yaqltypes.String())
    @inject_format
    def error(__self, __yaql_format_function, __message, *args, **kwargs):
        __self._log(__self._underlying_logger.error,
                    __yaql_format_function, __message, args, kwargs)

    @specs.parameter('_Logger__message', yaqltypes.String())
    @inject_format
    def critical(__self, __yaql_format_function,
                 __message, *args, **kwargs):
        __self._log(__self._underlying_logger.critical,
                    __yaql_format_function, __message, args, kwargs)

    @specs.parameter('_Logger__message', yaqltypes.String())
    @inject_format
    def exception(__self, __yaql_format_function,
                  __exc, __message, *args, **kwargs):
        """Print error message and stacktrace"""
        stack_trace_message = u'\n'.join([
            __self._format_without_exceptions(
                __yaql_format_function, __message, args, kwargs),
            __exc['stackTrace']().toString()
        ])
        __self._underlying_logger.error(stack_trace_message)

    def _format_without_exceptions(self, format_function,
                                   message, args, kwargs):
        """Wrap YAQL function "format" to suppress exceptions

        Wrap YAQL function "format" to suppress exceptions
        that may be raised when message cannot be formatted
        due to invalid parameters provided
        We do not want to break program workflow
        even when formatting parameters are incorrect
        """
        try:
            message = format_function(message, *args, **kwargs)
        except (IndexError, KeyError):
            # NOTE(akhivin): we do not want break program workflow
            # even formatting parameters are incorrect
            self._underlying_logger.warning(
                u'Can not format string: {0}'.format(message))
        return message

    def _log(self, log_function, yaql_format_function, message, args, kwargs):
        log_function(
            self._format_without_exceptions(
                yaql_format_function, message, args, kwargs))



#    Copyright (c) 2015 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc


class MuranoPackageLoader(object, metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def load_package(self, package_name, version_spec):
        pass

    @abc.abstractmethod
    def load_class_package(self, class_name, version_spec):
        pass

    @abc.abstractmethod
    def register_package(self, package):
        pass

    @abc.abstractmethod
    def import_fixation_table(self, fixations):
        pass

    @abc.abstractmethod
    def export_fixation_table(self):
        pass

    @abc.abstractmethod
    def compact_fixation_table(self):
        pass



# Copyright 2011 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from keystoneauth1 import loading as ks_loading
from oslo_config import cfg
from oslo_log import log as logging
from oslo_middleware import cors
from oslo_policy import opts

from murano.common.i18n import _
from murano import version

paste_deploy_opts = [
    cfg.StrOpt('flavor', help='Paste flavor'),
    cfg.StrOpt('config_file', help='Path to Paste config file'),
]

bind_opts = [
    cfg.HostAddressOpt('bind-host', default='0.0.0.0',
                       help='Address to bind the Murano API server to.'),
    cfg.PortOpt('bind-port',
                default=8082,
                help='Port the bind the Murano API server to.'),
]

rabbit_opts = [
    cfg.HostAddressOpt('host', default='localhost',
                       help='The RabbitMQ broker address which used for '
                            'communication with Murano guest agents.'),

    cfg.PortOpt('port',
                default=5672,
                help='The RabbitMQ broker port.'),

    cfg.StrOpt('login', default='guest',
               help='The RabbitMQ login.'),

    cfg.StrOpt('password', default='guest', secret=True,
               help='The RabbitMQ password.'),

    cfg.StrOpt('virtual_host', default='/',
               help='The RabbitMQ virtual host.'),

    cfg.BoolOpt('ssl', default=False,
                help='Boolean flag to enable SSL communication through the '
                'RabbitMQ broker between murano-engine and guest agents.'),

    cfg.StrOpt('ssl_version',
               default='',
               help='SSL version to use (valid only if SSL enabled). '
                    'Valid values are TLSv1 and SSLv23. SSLv2, SSLv3, '
                    'TLSv1_1, and TLSv1_2 may be available on some '
                    'distributions.'),

    cfg.StrOpt('ca_certs', default='',
               help='SSL cert file (valid only if SSL enabled).'),

    cfg.BoolOpt('insecure', default=False,
                help='This option explicitly allows Murano to perform '
                     '"insecure" SSL connections to RabbitMQ'),
]

heat_opts = [
    cfg.StrOpt('url', help='Optional heat endpoint override'),

    cfg.StrOpt('endpoint_type', default='publicURL',
               help='Heat endpoint type.'),

    cfg.ListOpt('stack_tags', default=['murano'],
                help='List of tags to be assigned to heat stacks created '
                     'during environment deployment.')
]

mistral_opts = [
    cfg.StrOpt('url', help='Optional mistral endpoint override'),

    cfg.StrOpt('endpoint_type', default='publicURL',
               help='Mistral endpoint type.'),

    cfg.StrOpt('service_type', default='workflowv2',
               help='Mistral service type.')
]

neutron_opts = [
    cfg.StrOpt('url', help='Optional neutron endpoint override'),

    cfg.StrOpt('endpoint_type', default='publicURL',
               help='Neutron endpoint type.')
]

murano_opts = [
    cfg.StrOpt('url', help='Optional murano url in format '
                           'like http://0.0.0.0:8082 used by Murano engine'),

    cfg.StrOpt('endpoint_type', default='publicURL',
               help='Murano endpoint type used by Murano engine.'),

    cfg.ListOpt('enabled_plugins',
                help="List of enabled Extension Plugins. "
                     "Remove or leave commented to enable all installed "
                     "plugins."),

    cfg.IntOpt('package_size_limit', default=5,
               help='Maximum application package size, Mb',
               deprecated_group='packages_opts'),

    cfg.IntOpt('limit_param_default', default=20,
               help='Default value for package pagination in API.',
               deprecated_group='packages_opts'),

    cfg.IntOpt('api_limit_max', default=100,
               help='Maximum number of packages to be returned in a single '
                    'pagination request',
               deprecated_group='packages_opts'),

    cfg.IntOpt('api_workers',
               help=_('Number of API workers')),

    cfg.IntOpt('dsl_iterators_limit', default=2000,
               help=_('Maximum number of elements that can be iterated per '
                      'object type.')),

]

networking_opts = [
    cfg.IntOpt('max_environments', default=250,
               help='Maximum number of environments that use a single router '
               'per tenant'),

    cfg.IntOpt('max_hosts', default=250,
               help='Maximum number of VMs per environment'),

    cfg.HostAddressOpt('env_ip_template', default='10.0.0.0',
                       help='Template IP address for generating environment '
                            'subnet cidrs'),

    cfg.ListOpt('default_dns', default=[],
                help='List of default DNS nameservers to be assigned to '
                     'created Networks'),

    cfg.StrOpt('external_network', default='ext-net',
               help='ID or name of the external network for routers '
                    'to connect to'),

    cfg.StrOpt('router_name', default='murano-default-router',
               help='Name of the router that going to be used in order to '
                    'join all networks created by Murano'),

    cfg.BoolOpt('create_router', default=True,
                help='This option will create a router when one with '
                     '"router_name" does not exist'),

    cfg.StrOpt('network_config_file', default='netconfig.yaml',
               help='If provided networking configuration will be taken '
                    'from this file'),

    cfg.StrOpt('driver', choices=['neutron', 'nova'],
               help='Network driver to use. Options are neutron or nova.'
                    'If not provided, the driver will be detected.'),
]

stats_opts = [
    cfg.IntOpt('period', default=5,
               help=_('Statistics collection interval in minutes.'
                      'Default value is 5 minutes.')),
    cfg.IntOpt('env_audit_period', default=60,
               help=_('Environment audit interval in minutes. '
                      'Default value is 60 minutes.')),
    cfg.BoolOpt('env_audit_enabled', default=True,
                help=_('Whether environment audit events enabled'))
]

engine_opts = [
    cfg.BoolOpt('disable_murano_agent', default=False,
                help=_('Disallow the use of murano-agent')),
    cfg.StrOpt('class_configs', default='/etc/murano/class-configs',
               help=_('Path to class configuration files')),
    cfg.BoolOpt('use_trusts', default=True,
                help=_("Create resources using trust token rather "
                       "than user's token")),
    cfg.BoolOpt('enable_model_policy_enforcer', default=False,
                help=_('Enable model policy enforcer using Congress')),
    cfg.IntOpt('agent_timeout', default=3600,
               help=_('Time for waiting for a response from murano agent '
                      'during the deployment')),
    cfg.IntOpt('engine_workers',
               deprecated_opts=[cfg.DeprecatedOpt('workers',
                                                  group='engine')],
               help=_('Number of engine workers')),

    cfg.ListOpt('load_packages_from', default=[],
                help=_('List of directories to load local packages from. '
                       'If not provided, packages will be loaded only API'),
                deprecated_group='packages_opts'),

    cfg.StrOpt('packages_cache',
               help='Location (directory) for Murano package cache.',
               deprecated_group='packages_opts'),

    cfg.BoolOpt('enable_packages_cache', default=True,
                help=_('Enables murano-engine to persist on disk '
                       'packages downloaded during deployments. '
                       'The packages would be re-used for consequent '
                       'deployments.'),
                deprecated_group='packages_opts'),

    cfg.StrOpt('packages_service', default='murano',
               help=_('The service to store murano packages: murano (stands '
                      'for legacy behavior using murano-api) or glance '
                      '(stands for glance-glare artifact service)'),
               deprecated_group='packages_opts'),

    cfg.StrOpt('signing_key', default='~/.ssh/id_rsa',
               help=_('Path to RSA key for agent message signing')),

    cfg.StrOpt('agent_source', default='murano-agent',
               help=_('pip URL/package spec for murano-agent')),
]

# TODO(sjmc7): move into engine opts?
metadata_dir = [
    cfg.StrOpt('metadata-dir', default='./meta',
               help='Metadata dir')
]

glare_opts = [
    cfg.StrOpt('url', help='Optional glare url in format '
                           'like http://0.0.0.0:9494 used by Glare API',
               deprecated_group='glance'),

    cfg.StrOpt('endpoint_type', default='publicURL',
               help='Glare endpoint type.',
               deprecated_group='glance')
]

glance_opts = [
    cfg.StrOpt('url', help='Optional glance endpoint override'),

    cfg.StrOpt('endpoint_type', default='publicURL',
               help='Glance endpoint type.')
]

file_server = [
    cfg.StrOpt('file_server', default='',
               help='Set a file server.')
]

home_region = cfg.StrOpt(
    'home_region',
    help="Default region name used to get services endpoints.")

# Unfortuntely we cannot use murano_auth.auth_url, since it
# is private to the actual authentication plugin used.
murano_auth_opts = [
    cfg.StrOpt('www_authenticate_uri',
               help='Identity API endpoint for authenticating with tokens.')]


CONF = cfg.CONF
CONF.register_opts(paste_deploy_opts, group='paste_deploy')
CONF.register_cli_opts(bind_opts)
CONF.register_opts(rabbit_opts, group='rabbitmq')
CONF.register_opts(heat_opts, group='heat')
CONF.register_opts(mistral_opts, group='mistral')
CONF.register_opts(neutron_opts, group='neutron')
CONF.register_opts(murano_opts, group='murano')
CONF.register_opts(engine_opts, group='engine')
CONF.register_opts(file_server)
CONF.register_opt(home_region)
CONF.register_cli_opts(metadata_dir)
CONF.register_opts(stats_opts, group='stats')
CONF.register_opts(networking_opts, group='networking')
CONF.register_opts(glare_opts, group='glare')
CONF.register_opts(glance_opts, group='glance')
CONF.register_opts(murano_auth_opts, group='murano_auth')
ks_loading.register_auth_conf_options(CONF, group='murano_auth')


for group in ('heat', 'mistral', 'neutron', 'glance', 'glare',
              'murano', 'murano_auth'):
    ks_loading.register_session_conf_options(
        CONF,
        group=group,
        deprecated_opts={
            'cafile': [cfg.DeprecatedOpt('cacert', group),
                       cfg.DeprecatedOpt('ca_file', group)],
            'certfile': [cfg.DeprecatedOpt('cert_file', group)],
            'keyfile': [cfg.DeprecatedOpt('key_file', group)]

        })


def parse_args(args=None, usage=None, default_config_files=None):
    logging.register_options(CONF)
    CONF(args=args,
         project='murano',
         version=version.version_string,
         usage=usage,
         default_config_files=default_config_files)


def set_lib_defaults():
    """Update default value for configuration options from other namespace.

    Example, oslo lib config options. This is needed for
    config generator tool to pick these default value changes.
    https://docs.openstack.org/oslo.config/latest/cli/
    generator.html#modifying-defaults-from-other-namespaces
    """

    set_middleware_defaults()

    # TODO(gmann): Remove setting the default value of config policy_file
    # once oslo_policy change the default value to 'policy.yaml'.
    # https://github.com/openstack/oslo.policy/blob/a626ad12fe5a3abd49d70e3e5b95589d279ab578/oslo_policy/opts.py#L49
    opts.set_defaults(CONF, 'policy.yaml')


def set_middleware_defaults():
    """Update default configuration options for oslo.middleware."""

    cors.set_defaults(
        allow_headers=['X-Auth-Token',
                       'X-Openstack-Request-Id',
                       'X-Configuration-Session',
                       'X-Roles',
                       'X-User-Id',
                       'X-Tenant-Id'],
        expose_headers=['X-Auth-Token',
                        'X-Openstack-Request-Id',
                        'X-Configuration-Session',
                        'X-Roles',
                        'X-User-Id',
                        'X-Tenant-Id'],
        allow_methods=['GET',
                       'PUT',
                       'POST',
                       'DELETE',
                       'PATCH']
    )



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import os
from unittest import mock

from oslo_config import cfg
from paste import deploy

from murano.common import app_loader
from murano.common import config  # noqa
from murano.tests.unit import base

CONF = cfg.CONF


class AppLoaderTest(base.MuranoTestCase):
    def setUp(self):
        super(AppLoaderTest, self).setUp()
        self.override_config('flavor', 'myflavor', 'paste_deploy')
        self.override_config('config_file', 'path/to/myapp-paste.ini',
                             'paste_deploy')
        CONF.config_file = ['myapp.conf']
        CONF.prog = 'myapp'
        CONF.find_file = mock.MagicMock(return_value='path/to/myapp-paste.ini')

    @mock.patch.object(deploy, 'loadapp', return_value=mock.sentinel.myapp)
    def _test_load_paste_app(self, mock_loadapp,
                             appname='myapp',
                             fullname='myapp-myflavor',
                             path='path/to/myapp-paste.ini'):
        expected_config_path = 'config:%s/%s' % (os.path.abspath('.'), path,)

        app = app_loader.load_paste_app(appname)

        mock_loadapp.assert_called_with(expected_config_path, name=fullname)
        self.assertEqual(mock.sentinel.myapp, app)

    def test_load_paste_app(self):
        self._test_load_paste_app()

    def test_load_paste_app_no_name(self):
        self._test_load_paste_app(appname=None)

    def test_load_paste_app_no_flavor(self):
        self.override_config('flavor', None, 'paste_deploy')
        self._test_load_paste_app(fullname='myapp')

    def test_load_paste_app_no_pastedep_cfg_opt(self):
        self.override_config('config_file', None, 'paste_deploy')

        self._test_load_paste_app()

    def test_load_paste_app_no_pastedep_cfg_opt_and_cfg_opt(self):
        self.override_config('config_file', None, 'paste_deploy')
        CONF.config_file = []

        self._test_load_paste_app()
        CONF.find_file.assert_called_with('myapp-paste.ini')

    def test_load_paste_app_no_cfg_at_all(self):
        self.override_config('config_file', None, 'paste_deploy')
        CONF.find_file.return_value = None

        self.assertRaises(RuntimeError, app_loader.load_paste_app, 'myapp')

    def test_load_paste_app_deploy_error(self):
        deploy.loadapp = mock.MagicMock()
        deploy.loadapp.side_effect = LookupError('Oops')

        self.assertRaises(RuntimeError, app_loader.load_paste_app, 'myapp')



# Copyright (c) 2015 OpenStack Foundation.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import murano.policy.modify.actions.default_actions as da
import murano.tests.unit.dsl.foundation.object_model as om
import murano.tests.unit.dsl.foundation.test_case as tc


class ModifyActionTestCase(tc.DslTestCase):
    def setUp(self):
        super(ModifyActionTestCase, self).setUp()
        self._list_member = om.Object('SampleClass2', class2Property='string2')
        self._dict_member = om.Object('SampleClass2', class2Property='string2')
        self._runner = self.new_runner(
            om.Object(
                'ModelExamples',
                sampleClass=om.Object(
                    'SampleClass1',
                    stringProperty='string1',
                    dictProperty={1: 'a', 2: 'b'},
                    dictClassProperty={
                        'key': self._dict_member},
                    classProperty=[self._list_member])))
        self._obj = self._runner.root


class TestRemoveObjectAction(ModifyActionTestCase):
    def test_remove(self):
        with self._runner.session():
            self.assertIsNotNone(self._obj.get_property('sampleClass'))
            object_id = self._obj.get_property('sampleClass').object_id
            da.RemoveObjectAction(object_id=object_id).modify(self._obj)
            self.assertIsNone(self._obj.get_property('sampleClass'))

    def test_remove_from_list(self):
        with self._runner.session():
            self.assertEqual(1, len(
                self._obj.get_property('sampleClass')
                    .get_property('classProperty')))
            da.RemoveObjectAction(object_id=self._list_member.id).modify(
                self._obj)
            self.assertEqual(0, len(
                self._obj.get_property('sampleClass')
                    .get_property('classProperty')))
            self.assertNotIn(self._list_member.id, repr(self._obj))

    def test_remove_from_dict(self):
        with self._runner.session():
            self.assertEqual(1, len(
                self._obj.get_property('sampleClass')
                    .get_property('dictClassProperty')))
            da.RemoveObjectAction(object_id=self._dict_member.id).modify(
                self._obj)
            self.assertEqual(0, len(
                self._obj.get_property('sampleClass')
                    .get_property('dictClassProperty')))
            self.assertNotIn(self._dict_member.id, repr(self._obj))

    def test_remove_not_exists(self):
        with self._runner.session():
            action = da.RemoveObjectAction(object_id='not_exists')
            self.assertRaises(ValueError, action.modify, self._obj)


class TestSetPropertyAction(ModifyActionTestCase):
    def test_set_str(self):
        self._test_set_value('stringProperty', 'test_string', 'test_string')

    def test_set_number(self):
        self._test_set_value('numberProperty', '15', 15)
        self._test_set_value('numberProperty', '50', 50)
        self._test_set_value('numberProperty', 40, 40)
        self._test_set_value('numberProperty', '-5', -5)

    def test_set_boolean(self):
        self._test_set_value('boolProperty', True, True)
        self._test_set_value('boolProperty', False, False)

    def test_set_dict(self):
        self._test_set_value('dictProperty', {1: 'a'}, {1: 'a'})
        self._test_set_value('dictProperty', {1: 'b', 2: 'c'},
                             {1: 'b', 2: 'c'})

    def _test_set_value(self, property, raw_input, expected):
        with self._runner.session():
            sample = self._obj.get_property('sampleClass')
            da.SetPropertyAction(sample.object_id, prop_name=property,
                                 value=raw_input).modify(self._obj)
            self.assertEqual(expected, sample.get_property(property))


class TestRemoveRelationAction(ModifyActionTestCase):
    def test_remove(self):
        with self._runner.session():
            self.assertIsNotNone(self._obj.get_property('sampleClass'))
            da.RemoveRelationAction(self._obj.object_id,
                                    prop_name='sampleClass').modify(self._obj)
            self.assertIsNone(self._obj.get_property('sampleClass'))


class TestAddRelationAction(ModifyActionTestCase):
    def test_add(self):
        with self._runner.session():
            sample = self._obj.get_property('sampleClass')
            self.assertIsNone(self._obj.get_property('anotherSampleClass'))
            da.AddRelationAction(source_id=self._obj.object_id,
                                 relation='anotherSampleClass',
                                 target_id=sample.object_id).modify(self._obj)
            self.assertIsNotNone(self._obj.get_property('anotherSampleClass'))
            rel_target = self._obj.get_property('anotherSampleClass').object_id
            self.assertEqual(sample.object_id, rel_target)


class TestAddObjectAction(ModifyActionTestCase):
    def test_add_object(self):
        with self._runner.session():
            self._obj.set_property('sampleClass', None)
            self.assertIsNone(self._obj.get_property('sampleClass'))
            da.AddObjectAction(
                self._obj.object_id,
                'sampleClass',
                'SampleClass1',
                {'stringProperty': 'test_add_obj'}).modify(self._obj)
            self.assertIsNotNone(self._obj.get_property('sampleClass'))
            self.assertEqual('test_add_obj',
                             self._obj.get_property('sampleClass')
                             .get_property('stringProperty'))



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import uuid

from oslo_db import exception as db_exception
from webob import exc

from murano.db.catalog import api
from murano.tests.unit import base
from murano.tests.unit import utils


class CatalogDBTestCase(base.MuranoWithDBTestCase):

    def setUp(self):
        super(CatalogDBTestCase, self).setUp()
        self.tenant_id = str(uuid.uuid4())
        self.tenant_id_2 = str(uuid.uuid4())
        self.context = utils.dummy_context(tenant_id=self.tenant_id)
        self.context_2 = utils.dummy_context(tenant_id=self.tenant_id_2)

        self.context_admin = utils.dummy_context(tenant_id=self.tenant_id)
        self.context_admin.is_admin = True

    def _create_categories(self):
        api.category_add('cat1')
        api.category_add('cat2')

    def _stub_package(self, **kwargs):
        base = {
            'archive': b"archive blob here",
            'fully_qualified_name': 'com.example.package',
            'type': 'class',
            'author': 'OpenStack',
            'name': 'package',
            'enabled': True,
            'description': 'some text',
            'is_public': False,
            'tags': ['tag1', 'tag2'],
            'logo': b"logo blob here",
            'ui_definition': '{}',
        }
        base.update(**kwargs)
        return base

    def get_change(self, op, path, value):
        return {
            'op': op,
            'path': path,
            'value': value
        }

    def test_order_by(self):
        pkgs = []
        for dummy in range(10):
            package = api.package_upload(self._stub_package(
                name=str(uuid.uuid4()),
                fully_qualified_name=str(uuid.uuid4())), self.tenant_id)
            pkgs.append(package)

        pkg_created = [pkg.id for pkg in sorted(
            pkgs, key=lambda _pkg: _pkg.created)]
        pkg_name = [pkg.id for pkg in sorted(
            pkgs, key=lambda _pkg: _pkg.name)]
        pkg_fqn = [pkg.id for pkg in sorted(
            pkgs, key=lambda _pkg: _pkg.fully_qualified_name)]

        for order, pkg_ids in zip(['created', 'name', 'fqn'],
                                  [pkg_created, pkg_name, pkg_fqn]):
            res = api.package_search(
                {'order_by': [order]}, self.context, limit=10)
            self.assertEqual(10, len(res))
            self.assertEqual(pkg_ids, [r.id for r in res])

    def test_order_by_compound(self):
        pkgs_a, pkgs_z = [], []
        for _ in range(5):
            package = api.package_upload(self._stub_package(
                name='z',
                fully_qualified_name=str(uuid.uuid4())), self.tenant_id)
            pkgs_z.append(package)
        for _ in range(5):
            package = api.package_upload(self._stub_package(
                name='a',
                fully_qualified_name=str(uuid.uuid4())), self.tenant_id)
            pkgs_a.append(package)

        # sort pkg ids by pkg created
        pkg_a_id = [pkg.id for pkg in sorted(
            pkgs_a, key=lambda _pkg: _pkg.created)]
        pkg_z_id = [pkg.id for pkg in sorted(
            pkgs_z, key=lambda _pkg: _pkg.created)]

        res = api.package_search(
            {'order_by': ['name', 'created']}, self.context, limit=10)
        self.assertEqual(10, len(res))
        self.assertEqual(pkg_a_id + pkg_z_id, [r.id for r in res])

    def test_pagination_backwards(self):
        """Tests backwards pagination

        Creates 10 packages with unique names and iterates backwards,
        checking that package order is correct.
        """
        pkgs = []
        for dummy in range(10):
            package = api.package_upload(self._stub_package(
                name=str(uuid.uuid4()),
                fully_qualified_name=str(uuid.uuid4())), self.tenant_id)
            pkgs.append(package)

        # sort pkg ids by pkg name
        pkg_ids = [pkg.id for pkg in sorted(pkgs, key=lambda _pkg: _pkg.name)]

        res = api.package_search({}, self.context, limit=10)
        self.assertEqual(10, len(res))
        self.assertEqual(pkg_ids, [r.id for r in res])
        marker = res[-1].id

        res = api.package_search(
            {'marker': marker,
             'sort_dir': 'desc'}, self.context, limit=5)
        self.assertEqual(5, len(res))
        self.assertEqual(list(reversed(pkg_ids[4:9])),
                         [r.id for r in res])
        marker = res[-1].id

        res = api.package_search(
            {'marker': marker,
             'sort_dir': 'desc'}, self.context, limit=5)
        self.assertEqual(4, len(res))
        self.assertEqual(list(reversed(pkg_ids[0:4])),
                         [r.id for r in res])
        marker = res[-1].id

        res = api.package_search(
            {'marker': marker,
             'sort_dir': 'desc'}, self.context, limit=5)
        self.assertEqual(0, len(res))

    def test_pagination(self):
        """Tests that package order is correct

        Creates 10 packages with unique names and iterates through them,
        checking that package order is correct.
        """

        pkgs = []
        for dummy in range(10):
            package = api.package_upload(self._stub_package(
                name=str(uuid.uuid4()),
                fully_qualified_name=str(uuid.uuid4())), self.tenant_id)
            pkgs.append(package)

        # sort pkg ids by pkg name
        pkg_ids = [pkg.id for pkg in sorted(pkgs, key=lambda _pkg: _pkg.name)]

        res = api.package_search({}, self.context, limit=4)
        self.assertEqual(4, len(res))
        self.assertEqual(pkg_ids[0:4], [r.id for r in res])
        marker = res[-1].id

        res = api.package_search({'marker': marker}, self.context, limit=4)
        self.assertEqual(4, len(res))
        self.assertEqual(pkg_ids[4:8], [r.id for r in res])
        marker = res[-1].id

        res = api.package_search({'marker': marker}, self.context, limit=4)
        self.assertEqual(2, len(res))
        self.assertEqual(pkg_ids[8:10], [r.id for r in res])
        marker = res[-1].id

        res = api.package_search({'marker': marker}, self.context, limit=4)
        self.assertEqual(0, len(res))

    def test_pagination_loops_through_names(self):
        """Tests that packages with same display name are not skipped

        Creates 10 packages with the same display name and iterates
        through them, checking that package are not skipped.
        """

        for dummy in range(10):
            api.package_upload(
                self._stub_package(name='test',
                                   fully_qualified_name=str(uuid.uuid4())),
                self.tenant_id)
        res = api.package_search({}, self.context, limit=4)
        self.assertEqual(4, len(res))
        marker = res[-1].id

        res = api.package_search({'marker': marker}, self.context, limit=4)
        self.assertEqual(4, len(res))
        marker = res[-1].id

        res = api.package_search({'marker': marker}, self.context, limit=4)
        self.assertEqual(2, len(res))
        marker = res[-1].id

        res = api.package_search({'marker': marker}, self.context, limit=4)
        self.assertEqual(0, len(res))

    def test_package_search_search_order(self):
        pkg1 = api.package_upload(
            self._stub_package(
                fully_qualified_name=str(uuid.uuid4()),
                name='mysql',
                description='awcloud'),
            self.tenant_id)
        pkg2 = api.package_upload(
            self._stub_package(
                fully_qualified_name=str(uuid.uuid4()),
                name='awcloud',
                description='mysql'),
            self.tenant_id)
        api.package_upload(
            self._stub_package(
                tags=[],
                fully_qualified_name=str(uuid.uuid4())),
            self.tenant_id)

        res = api.package_search(
            {'search': 'mysql'}, self.context)
        self.assertEqual(2, len(res))
        self.assertEqual(pkg1.name, res[0].name)
        self.assertEqual(pkg2.description, res[1].description)

    def test_package_search_search(self):
        pkg1 = api.package_upload(
            self._stub_package(
                fully_qualified_name=str(uuid.uuid4())), self.tenant_id)
        pkg2 = api.package_upload(
            self._stub_package(
                tags=[],
                fully_qualified_name=str(uuid.uuid4())), self.tenant_id)

        res = api.package_search(
            {'search': 'tag1'}, self.context)
        self.assertEqual(1, len(res))
        res = api.package_search(
            {'search': pkg1.fully_qualified_name}, self.context)
        self.assertEqual(1, len(res))
        res = api.package_search(
            {'search': pkg2.fully_qualified_name}, self.context)
        self.assertEqual(1, len(res))
        res = api.package_search(
            {'search': 'not_a_valid_uuid'}, self.context)
        self.assertEqual(0, len(res))

        res = api.package_search(
            {'search': 'some text'}, self.context)
        self.assertEqual(2, len(res))

    def test_package_search_tags(self):
        api.package_upload(
            self._stub_package(
                fully_qualified_name=str(uuid.uuid4())), self.tenant_id)
        api.package_upload(
            self._stub_package(
                tags=[],
                fully_qualified_name=str(uuid.uuid4())), self.tenant_id)

        res = api.package_search(
            {'tag': ['tag1']}, self.context)
        self.assertEqual(1, len(res))
        res = api.package_search(
            {'tag': ['tag2']}, self.context)
        self.assertEqual(1, len(res))
        res = api.package_search(
            {'tag': ['tag3']}, self.context)
        self.assertEqual(0, len(res))

    def test_package_search_type(self):
        api.package_upload(
            self._stub_package(
                type="Application",
                fully_qualified_name=str(uuid.uuid4())), self.tenant_id)
        api.package_upload(
            self._stub_package(
                type="Library",
                fully_qualified_name=str(uuid.uuid4())), self.tenant_id)

        res = api.package_search(
            {'type': 'Library'}, self.context)
        self.assertEqual(1, len(res))
        res = api.package_search(
            {'type': 'Application'}, self.context)
        self.assertEqual(1, len(res))

    def test_package_search_disabled(self):
        api.package_upload(
            self._stub_package(
                is_public=True,
                enabled=True,
                fully_qualified_name=str(uuid.uuid4())), self.tenant_id)
        api.package_upload(
            self._stub_package(
                is_public=True,
                enabled=False,
                fully_qualified_name=str(uuid.uuid4())), self.tenant_id)

        res = api.package_search(
            {'include_disabled': 'false'}, self.context)
        self.assertEqual(1, len(res))
        res = api.package_search(
            {'include_disabled': 'true'}, self.context)
        self.assertEqual(2, len(res))

    def test_package_search_owned(self):
        api.package_upload(
            self._stub_package(
                is_public=True,
                fully_qualified_name=str(uuid.uuid4())), self.tenant_id)
        api.package_upload(
            self._stub_package(
                is_public=True,
                fully_qualified_name=str(uuid.uuid4())), self.tenant_id_2)

        res = api.package_search({'owned': 'true'}, self.context_admin)
        self.assertEqual(1, len(res))
        res = api.package_search({'owned': 'false'}, self.context_admin)
        self.assertEqual(2, len(res))

    def test_package_search_no_filters_catalog(self):
        res = api.package_search({}, self.context, catalog=True)
        self.assertEqual(0, len(res))

        api.package_upload(
            self._stub_package(
                is_public=True,
                fully_qualified_name=str(uuid.uuid4())), self.tenant_id)
        api.package_upload(
            self._stub_package(
                is_public=False,
                fully_qualified_name=str(uuid.uuid4())), self.tenant_id)

        api.package_upload(
            self._stub_package(
                is_public=True,
                fully_qualified_name=str(uuid.uuid4())), self.tenant_id_2)
        api.package_upload(
            self._stub_package(
                is_public=False,
                fully_qualified_name=str(uuid.uuid4())), self.tenant_id_2)

        # catalog=True should show public + mine
        res = api.package_search({}, self.context, catalog=True)
        self.assertEqual(3, len(res))

        res = api.package_search({}, self.context_admin, catalog=True)
        self.assertEqual(3, len(res))

    def test_package_search_no_filters(self):
        res = api.package_search({}, self.context)
        self.assertEqual(0, len(res))

        api.package_upload(
            self._stub_package(
                is_public=True,
                fully_qualified_name=str(uuid.uuid4())), self.tenant_id)
        api.package_upload(
            self._stub_package(
                is_public=False,
                fully_qualified_name=str(uuid.uuid4())), self.tenant_id)

        api.package_upload(
            self._stub_package(
                is_public=True,
                fully_qualified_name=str(uuid.uuid4())), self.tenant_id_2)
        api.package_upload(
            self._stub_package(
                is_public=False,
                fully_qualified_name=str(uuid.uuid4())), self.tenant_id_2)

        # I can only edit mine pkgs
        res = api.package_search({}, self.context)
        self.assertEqual(2, len(res))
        for pkg in res:
            self.assertEqual(self.tenant_id, pkg.owner_id)

        # Admin can see everything
        res = api.package_search({}, self.context_admin)
        self.assertEqual(4, len(res))

    def test_list_empty_categories(self):
        res = api.category_get_names()
        self.assertEqual(0, len(res))

    def test_add_list_categories(self):
        self._create_categories()

        res = api.categories_list()
        self.assertEqual(2, len(res))

        for cat in res:
            self.assertIsNotNone(cat.id)
            self.assertTrue(cat.name.startswith('cat'))

    def test_package_upload(self):
        self._create_categories()
        values = self._stub_package()

        package = api.package_upload(values, self.tenant_id)

        self.assertIsNotNone(package.id)
        for k in values.keys():
            self.assertEqual(values[k], package[k])

    def test_package_fqn_is_unique(self):
        self._create_categories()
        values = self._stub_package()

        api.package_upload(values, self.tenant_id)
        self.assertRaises(db_exception.DBDuplicateEntry,
                          api.package_upload, values, self.tenant_id)

    def test_package_delete(self):
        values = self._stub_package()
        package = api.package_upload(values, self.tenant_id)

        api.package_delete(package.id, self.context)

        self.assertRaises(exc.HTTPNotFound,
                          api.package_get, package.id, self.context)

    def test_package_upload_to_different_tenants_with_same_fqn(self):
        values = self._stub_package()

        api.package_upload(values, self.tenant_id)
        api.package_upload(values, self.tenant_id_2)

    def test_package_upload_public_public_fqn_violation(self):
        values = self._stub_package(is_public=True)
        api.package_upload(values, self.tenant_id)
        values = self._stub_package(is_public=True)
        self.assertRaises(exc.HTTPConflict, api.package_upload,
                          values, self.tenant_id_2)

    def test_package_upload_public_private_no_fqn_violation(self):
        values = self._stub_package(is_public=True)
        api.package_upload(values, self.tenant_id)
        values = self._stub_package(is_public=False)
        api.package_upload(values, self.tenant_id_2)

    def test_package_upload_private_public_no_fqn_violation(self):
        values = self._stub_package()
        api.package_upload(values, self.tenant_id)
        values = self._stub_package(is_public=True)
        api.package_upload(values, self.tenant_id_2)

    def test_class_name_is_unique(self):
        value = self._stub_package(class_definitions=('foo', 'bar'))
        api.package_upload(value, self.tenant_id)
        value = self._stub_package(class_definitions=('bar', 'baz'),
                                   fully_qualified_name='com.example.package2')
        self.assertRaises(exc.HTTPConflict, api.package_upload, value,
                          self.tenant_id)

    def test_class_name_uniqueness_not_enforced_in_different_tenants(self):
        value = self._stub_package(class_definitions=('foo', 'bar'))
        api.package_upload(value, self.tenant_id)
        value = self._stub_package(class_definitions=('foo', 'bar'),
                                   fully_qualified_name='com.example.package2')
        api.package_upload(value, self.tenant_id_2)

    def test_class_name_public_public_violation(self):
        value = self._stub_package(class_definitions=('foo', 'bar'),
                                   is_public=True)
        api.package_upload(value, self.tenant_id)
        value = self._stub_package(class_definitions=('foo', 'bar'),
                                   is_public=True,
                                   fully_qualified_name='com.example.package2')
        self.assertRaises(exc.HTTPConflict, api.package_upload,
                          value, self.tenant_id_2)

    def test_class_name_public_private_no_violation(self):
        value = self._stub_package(class_definitions=('foo', 'bar'),
                                   is_public=True)
        api.package_upload(value, self.tenant_id)
        value = self._stub_package(class_definitions=('foo', 'bar'),
                                   is_public=False,
                                   fully_qualified_name='com.example.package2')
        api.package_upload(value, self.tenant_id_2)

    def test_class_name_private_public_no_violation(self):
        value = self._stub_package(class_definitions=('foo', 'bar'),
                                   is_public=False)
        api.package_upload(value, self.tenant_id)
        value = self._stub_package(class_definitions=('foo', 'bar'),
                                   is_public=True,
                                   fully_qualified_name='com.example.package2')
        api.package_upload(value, self.tenant_id_2)

    def test_package_make_public(self):
        id = api.package_upload(self._stub_package(), self.tenant_id).id
        patch = self.get_change('replace', ['is_public'], True)
        api.package_update(id, [patch], self.context)
        package = api.package_get(id, self.context)
        self.assertTrue(package.is_public)

    def test_package_update_public_public_fqn_violation(self):
        id1 = api.package_upload(self._stub_package(), self.tenant_id).id
        id2 = api.package_upload(self._stub_package(), self.tenant_id_2).id
        patch = self.get_change('replace', ['is_public'], True)
        api.package_update(id1, [patch], self.context)
        self.assertRaises(exc.HTTPConflict, api.package_update,
                          id2, [patch], self.context_2)

    def test_package_update_public_public_class_name_violation(self):
        id1 = api.package_upload(self._stub_package(
            class_definitions=('foo', 'bar')), self.tenant_id).id
        id2 = api.package_upload(self._stub_package(
            class_definitions=('foo', 'bar'),
            fully_qualified_name='com.example.package2'), self.tenant_id_2).id
        patch = self.get_change('replace', ['is_public'], True)
        api.package_update(id1, [patch], self.context)
        self.assertRaises(exc.HTTPConflict, api.package_update,
                          id2, [patch], self.context_2)

    def test_category_paginate(self):
        """Paginate through a list of categories using limit and marker"""

        category_names = ['cat1', 'cat2', 'cat3', 'cat4', 'cat5']
        categories = []
        for name in category_names:
            categories.append(api.category_add(name))
        uuids = [c.id for c in categories]

        page = api.categories_list(limit=2)

        self.assertEqual(category_names[:2], [c.name for c in page])

        last = page[-1].id
        page = api.categories_list(limit=3, marker=last)
        self.assertEqual(category_names[2:5], [c.name for c in page])

        page = api.categories_list(marker=uuids[-1])
        self.assertEqual([], page)

        category_names.reverse()
        page = api.categories_list({'sort_dir': 'desc'})
        self.assertEqual(category_names, [c.name for c in page])

    def test_category_get_delete_error(self):
        category_id = 12
        self.assertRaises(exc.HTTPNotFound, api.category_get, category_id)
        self.assertRaises(exc.HTTPNotFound, api.category_delete, category_id)

    def test_get_categories_error(self):
        category_names = ['cat1', 'cat2', 'cat3', 'cat4', 'cat5']
        cat_session = None
        self.assertRaises(exc.HTTPBadRequest, api._get_categories,
                          category_names, cat_session)

    def test_authorize_package_delete_error(self):
        values = self._stub_package()
        package = api.package_upload(values, self.tenant_id)
        self.assertRaises(exc.HTTPForbidden, api._authorize_package, package,
                          self.context_2)
        self.assertRaises(exc.HTTPForbidden,
                          api.package_delete, package.id, self.context_2)
        id = package.id
        patch = self.get_change('replace', ['is_public'], False)
        api.package_update(id, [patch], self.context)
        self.assertRaises(exc.HTTPForbidden, api._authorize_package, package,
                          self.context_2, allow_public=True)



# Copyright 2011 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg
from oslo_log import log as logging
from oslo_middleware import cors

from murano.common.i18n import _
from murano import version


cfapi_opts = [
    cfg.StrOpt('tenant', default='admin',
               help=_('Project for service broker')),
    cfg.HostAddressOpt('bind_host', default='localhost',
                       help=_('Host for service broker')),
    cfg.StrOpt('bind_port', default='8083',
               help=_('Port for service broker')),
    cfg.StrOpt('auth_url', default='localhost:5000',
               help=_('Authentication URL')),
    cfg.StrOpt('user_domain_name', default='default',
               help=_('Domain name of the user')),
    cfg.StrOpt('project_domain_name', default='default',
               help=_('Domain name of the project')),
    cfg.StrOpt('packages_service', default='murano',
               help=_('Package service which should be used by service'
                      ' broker'))]

CONF = cfg.CONF
CONF.register_opts(cfapi_opts, group='cfapi')


def parse_args(args=None, usage=None, default_config_files=None):
    logging.register_options(CONF)
    CONF(args=args,
         project='murano',
         version=version.version_string,
         usage=usage,
         default_config_files=default_config_files)


def set_middleware_defaults():
    """Update default configuration options for oslo.middleware."""
    cors.set_defaults(
        allow_headers=['X-Auth-Token',
                       'X-Openstack-Request-Id',
                       'X-Configuration-Session',
                       'X-Roles',
                       'X-User-Id',
                       'X-Tenant-Id'],
        expose_headers=['X-Auth-Token',
                        'X-Openstack-Request-Id',
                        'X-Configuration-Session',
                        'X-Roles',
                        'X-User-Id',
                        'X-Tenant-Id'],
        allow_methods=['GET',
                       'PUT',
                       'POST',
                       'DELETE',
                       'PATCH']
    )



#    Copyright (c) 2015 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import datetime as dt
from unittest import mock
import uuid

from oslo_utils import timeutils

from murano.db import models
from murano.db.services import environments
from murano.db import session as db_session
from murano.services import states
from murano.tests.unit import base
from murano.tests.unit import utils

OLD_VERSION = 0
LATEST_VERSION = 1


class TestEnvironmentServices(base.MuranoWithDBTestCase):
    def setUp(self):
        super(TestEnvironmentServices, self).setUp()
        self.environment = models.Environment(
            name='test_environment', tenant_id='test_tenant_id',
            version=LATEST_VERSION
        )

        self.env_services = environments.EnvironmentServices()

    def test_environment_ready_if_last_session_deployed_after_failed(self):
        """Test environment ready status

        If last session was deployed successfully and other session
        was failed - environment must have status "ready".

        Bug: #1413260
        """

        session = db_session.get_session()

        session.add(self.environment)

        now = timeutils.utcnow()

        session_1 = models.Session(
            environment=self.environment, user_id='test_user_id_1',
            version=OLD_VERSION,
            state=states.SessionState.DEPLOY_FAILURE,
            updated=now, description={}
        )
        session_2 = models.Session(
            environment=self.environment, user_id='test_user_id_2',
            version=LATEST_VERSION,
            state=states.SessionState.DEPLOYED,
            updated=now + dt.timedelta(minutes=1), description={}
        )
        session.add_all([session_1, session_2])
        session.flush()

        expected_status = states.EnvironmentStatus.READY
        actual_status = environments.EnvironmentServices.get_status(
            self.environment.id
        )

        self.assertEqual(expected_status, actual_status)

    @mock.patch("murano.db.services.environments.auth_utils")
    def test_get_network_driver(self, mock_authentication):
        self.tenant_id = str(uuid.uuid4())
        self.context = utils.dummy_context(tenant_id=self.tenant_id)

        driver_context = self.env_services.get_network_driver(self.context)
        self.assertEqual(driver_context, "neutron")

    def test_get_status(self):
        session = db_session.get_session()

        session.add(self.environment)

        now = timeutils.utcnow()

        session_1 = models.Session(
            environment=self.environment, user_id='test_user_id_1',
            version=OLD_VERSION,
            state=states.SessionState.DEPLOY_FAILURE,
            updated=now, description={}
        )

        session.add(session_1)
        session.flush()

        expected_status = states.EnvironmentStatus.DEPLOY_FAILURE
        self.assertEqual(expected_status, self.env_services.get_status(
            self.environment.id))

    def test_delete_failure_get_description(self):
        session = db_session.get_session()

        session.add(self.environment)

        now = timeutils.utcnow()

        session_1 = models.Session(
            environment=self.environment, user_id='test_user_id_1',
            version=OLD_VERSION,
            state=states.SessionState.DELETE_FAILURE,
            updated=now, description={}
        )

        session.add(session_1)
        session.flush()

        expected_status = states.EnvironmentStatus.DELETE_FAILURE
        self.assertEqual(expected_status, self.env_services.get_status(
            self.environment.id))

        env_id = self.environment.id
        description = (self.env_services.
                       get_environment_description(env_id,
                                                   session_id=None,
                                                   inner=False))
        self.assertEqual({}, description)



# Copyright (c) 2016 AT&T Corp
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from murano.services import actions
from murano.services import states
import murano.tests.unit.base as test_base


class TestActions(test_base.MuranoTestCase):

    def test_create_action_task(self):
        mock_action_name = 'test_action_name'
        mock_target_obj = '123'
        mock_args = {'param1': 'something', 'param2': 'something else'}
        mock_environment = mock.MagicMock(id='456', tenant_id='789')
        mock_description = {
            'Objects': {
                '?': {
                    'id': '456'
                },
                'applications': [],
                'services': ['service1', 'service2']
            },
            'project_id': 'XXX',
            'user_id': 'YYY'
        }
        mock_session = mock.MagicMock(description=mock_description)
        mock_context = mock.Mock(auth_token='test_token',
                                 tenant='test_tenant',
                                 user='test_user')
        expected_task = {
            'action': {
                'object_id': mock_target_obj,
                'method': mock_action_name,
                'args': mock_args
            },
            'model': {
                'Objects': {
                    '?': {
                        'id': mock_environment.id
                    },
                    'applications':
                        mock_session.description['Objects']['services']
                },
                'project_id': 'XXX',
                'user_id': 'YYY'
            },
            'token': 'test_token',
            'project_id': 'test_tenant',
            'user_id': 'test_user',
            'id': mock_environment.id
        }

        task = actions.ActionServices.create_action_task(mock_action_name,
                                                         mock_target_obj,
                                                         mock_args,
                                                         mock_environment,
                                                         mock_session,
                                                         mock_context)

        self.assertEqual(expected_task, task)

    @mock.patch('murano.services.actions.models')
    def test_update_task(self, mock_models):
        mock_models.Task = mock.MagicMock()
        mock_models.Status = mock.MagicMock()
        mock_action = [{}, {'name': 'test_action_name'}]
        mock_session = mock.MagicMock(environment_id='123',
                                      state=states.SessionState.OPENED,
                                      description={'Objects': ['o1', 'o2']})
        mock_task = {'action': 'test_action_name'}
        mock_unit = mock.MagicMock(__enter__=mock.MagicMock(),
                                   __exit__=mock.MagicMock())

        actions.ActionServices.update_task(mock_action, mock_session,
                                           mock_task, mock_unit)

        self.assertEqual(mock_session.environment_id,
                         mock_models.Task().environment_id)
        self.assertEqual(dict(mock_session.description['Objects']),
                         mock_models.Task().description)
        self.assertEqual(mock_task['action'],
                         mock_models.Task().action)
        self.assertIn(mock_action[1]['name'],
                      mock_models.Status().text)
        self.assertEqual('info', mock_models.Status().level)
        mock_models.Task().statuses.append.assert_called_once_with(
            mock_models.Status())
        self.assertEqual(2, mock_unit.add.call_count)
        expected_session = mock_session
        expected_session.state = states.SessionState.DEPLOYED
        mock_unit.add.assert_any_call(expected_session)
        mock_unit.add.assert_called_with(mock_models.Task())

    @mock.patch('murano.services.actions.rpc')
    @mock.patch('murano.services.actions.actions_db.update_task')
    @mock.patch('murano.services.actions.ActionServices.create_action_task')
    def test_submit_task(self, mock_create_action_task, mock_update_task,
                         mock_rpc):
        mock_task = mock.MagicMock()
        mock_create_action_task.return_value = mock_task
        mock_update_task.return_value = '123'
        mock_rpc.engine().handle_task = mock.MagicMock()

        test_action_name = 'test_action_name'
        test_target_obj = 'test_target_obj'
        test_args = 'test_args'
        test_environment = 'test_environment'
        test_session = 'test_session'
        context = mock.Mock()
        context.auth_token = 'test_token'
        context.tenant = 'test_tenant'
        context.user = 'test_user'
        test_unit = 'test_unit'

        task_id = actions.ActionServices.submit_task(test_action_name,
                                                     test_target_obj,
                                                     test_args,
                                                     test_environment,
                                                     test_session,
                                                     context,
                                                     test_unit)

        self.assertEqual('123', task_id)
        mock_create_action_task.assert_called_once_with(test_action_name,
                                                        test_target_obj,
                                                        test_args,
                                                        test_environment,
                                                        test_session,
                                                        context)
        mock_update_task.assert_called_once_with(test_action_name,
                                                 test_session, mock_task,
                                                 test_unit)
        mock_rpc.engine().handle_task.assert_called_once_with(mock_task)

    @mock.patch('murano.services.actions.actions_db.get_environment')
    @mock.patch('murano.services.actions.ActionServices.submit_task')
    def test_execute(self, mock_submit_task, mock_get_environment):
        mock_environment = mock.MagicMock()
        mock_task_id = 'test_task_id'
        mock_get_environment.return_value = mock_environment
        mock_submit_task.return_value = mock_task_id

        test_action_id = 'test_action_id'
        test_description = [{
            '?': {
                'id': 'test_obj_id',
                '_actions': {
                    test_action_id: {
                        'name': 'test_action_1',
                        'enabled': True
                    }
                }
            }
        }]
        test_session = mock.MagicMock(description=test_description)
        test_unit = 'test_unit'
        test_token = 'test_token'
        test_args = None
        expected_action_name = 'test_action_1'
        expected_target_obj = 'test_obj_id'

        task_id = actions.ActionServices.execute(test_action_id, test_session,
                                                 test_unit, test_token,
                                                 test_args)

        self.assertEqual(mock_task_id, task_id)
        mock_submit_task.assert_called_once_with(expected_action_name,
                                                 expected_target_obj,
                                                 {}, mock_environment,
                                                 test_session, test_token,
                                                 test_unit)

    @mock.patch('murano.services.actions.actions_db.get_environment')
    def test_execute_with_invalid_action_id(self, mock_get_environment):
        test_action_id = 'test_action_id'
        test_session = mock.MagicMock(description=[])

        with self.assertRaisesRegex(LookupError, 'Action is not found'):
            actions.ActionServices.execute(test_action_id, test_session,
                                           None, None, None)

    @mock.patch('murano.services.actions.actions_db.get_environment')
    def test_execute_with_disabled_action(self, mock_get_environment):
        test_action_id = 'test_action_id'
        test_description = [{
            '?': {
                'id': 'test_obj_id',
                '_actions': {
                    test_action_id: {
                        'name': 'test_action_1',
                        'enabled': False
                    }
                }
            }
        }]
        test_session = mock.MagicMock(description=test_description)

        with self.assertRaisesRegex(ValueError,
                                    'Cannot execute disabled action'):
            actions.ActionServices.execute(test_action_id, test_session,
                                           None, None, None)

    def test_get_result(self):
        mock_task = mock.MagicMock(result='test_result')
        mock_unit = mock.MagicMock()
        mock_unit.query().filter_by().first.return_value = mock_task
        result = actions.ActionServices.get_result('eid', 'tid', mock_unit)
        self.assertEqual(mock_task.result, result)

        mock_unit.query().filter_by().first.return_value = None
        task = actions.ActionServices.get_result('eid', 'tid', mock_unit)
        self.assertIsNone(task)
        mock_unit.query().filter_by.assert_any_call(id='tid',
                                                    environment_id='eid')



# coding: utf-8
# Copyright (c) 2015 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from murano.dsl import helpers
from murano.engine.system import logger
from murano.tests.unit.dsl.foundation import object_model as om
from murano.tests.unit.dsl.foundation import test_case


class TestLogger(test_case.DslTestCase):

    FORMAT_CALLS = [
        mock.call(mock.ANY, 'str', (), {}),
        mock.call(mock.ANY, u'тест', (), {}),
        mock.call(mock.ANY, 'str', (1,), {}),
        mock.call(mock.ANY, 'str {0}', ('message',), {}),
        mock.call(mock.ANY, 'str {message}', (), {'message': 'message'}),
        mock.call(mock.ANY, 'str {message}{0}', (), {})]

    LOG_CALLS = FORMAT_CALLS

    def setUp(self):
        super(TestLogger, self).setUp()
        self._runner = self.new_runner(om.Object('TestLogger'))
        self.package_loader.load_package('io.murano', None).register_class(
            logger.Logger)

    def test_create(self):
        logger_instance = self._runner.testCreate()
        self.assertTrue(
            helpers.is_instance_of(logger_instance, 'io.murano.system.Logger'),
            'Function should return io.murano.system.Logger instance')

    def _create_logger_mock(self):
        logger_instance = self._runner.testCreate()
        logger_ext = logger_instance.extension

        underlying_logger_mock = mock.MagicMock()
        logger_ext._underlying_logger = underlying_logger_mock
        logger_ext._underlying_logger.return_value = None

        format_mock = mock.MagicMock(return_value='format_mock')
        # do not verify number of conversions to string
        format_mock.__str__ = mock.MagicMock(return_value='format_mock')
        format_mock.__unicode__ = mock.MagicMock(return_value='format_mock')

        logger_ext._format_without_exceptions = format_mock

        return logger_instance, format_mock, underlying_logger_mock

    def test_trace(self):
        logger_instance, format_mock, underlying_logger_mock \
            = self._create_logger_mock()
        log_method = mock.MagicMock()

        underlying_logger_mock.trace = log_method
        self._runner.testTrace(logger_instance)

        format_mock.assert_has_calls(self.FORMAT_CALLS, any_order=False)
        self.assertEqual(log_method.call_count, len(self.LOG_CALLS))

    def test_debug(self):
        logger_instance, format_mock, underlying_logger_mock \
            = self._create_logger_mock()
        log_method = mock.MagicMock()

        underlying_logger_mock.debug = log_method
        self._runner.testDebug(logger_instance)

        format_mock.assert_has_calls(self.FORMAT_CALLS, any_order=False)
        self.assertEqual(log_method.call_count, len(self.LOG_CALLS))

    def test_info(self):
        logger_instance, format_mock, underlying_logger_mock \
            = self._create_logger_mock()
        log_method = mock.MagicMock()

        underlying_logger_mock.info = log_method
        self._runner.testInfo(logger_instance)

        format_mock.assert_has_calls(self.FORMAT_CALLS, any_order=False)
        self.assertEqual(log_method.call_count, len(self.LOG_CALLS))

    def test_warning(self):
        logger_instance, format_mock, underlying_logger_mock \
            = self._create_logger_mock()
        log_method = mock.MagicMock()

        underlying_logger_mock.warning = log_method
        self._runner.testWarning(logger_instance)

        format_mock.assert_has_calls(self.FORMAT_CALLS, any_order=False)
        self.assertEqual(log_method.call_count, len(self.LOG_CALLS))

    def test_error(self):
        logger_instance, format_mock, underlying_logger_mock \
            = self._create_logger_mock()
        log_method = mock.MagicMock()

        underlying_logger_mock.error = log_method
        self._runner.testError(logger_instance)

        format_mock.assert_has_calls(self.FORMAT_CALLS, any_order=False)
        self.assertEqual(log_method.call_count, len(self.FORMAT_CALLS))

    def test_critical(self):
        logger_instance, format_mock, underlying_logger_mock \
            = self._create_logger_mock()
        log_method = mock.MagicMock()

        underlying_logger_mock.critical = log_method
        self._runner.testCritical(logger_instance)

        format_mock.assert_has_calls(self.FORMAT_CALLS, any_order=False)
        self.assertEqual(log_method.call_count, len(self.LOG_CALLS))

    def test_exception(self):
        logger_instance, format_mock, underlying_logger_mock = \
            self._create_logger_mock()
        log_method = mock.MagicMock()

        underlying_logger_mock.error = log_method
        self._runner.testException(logger_instance)

        format_mock.assert_has_calls(self.FORMAT_CALLS, any_order=False)
        self.assertEqual(log_method.call_count, len(self.LOG_CALLS))



#    Copyright (c) 2013 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
SQLAlchemy models for murano data
"""
from oslo_db.sqlalchemy import models
from oslo_utils import timeutils
import sqlalchemy as sa
from sqlalchemy.ext import declarative
from sqlalchemy import orm as sa_orm

from murano.common import uuidutils
from murano.db.sqla import types as st


class TimestampMixin(object):
    __protected_attributes__ = set(["created", "updated"])

    created = sa.Column(sa.DateTime, default=timeutils.utcnow,
                        nullable=False)
    updated = sa.Column(sa.DateTime, default=timeutils.utcnow,
                        nullable=False, onupdate=timeutils.utcnow)

    def update(self, values):
        """dict.update() behaviour."""
        self.updated = timeutils.utcnow()
        super(TimestampMixin, self).update(values)

    def __setitem__(self, key, value):
        self.updated = timeutils.utcnow()
        super(TimestampMixin, self).__setitem__(key, value)


class _MuranoBase(models.ModelBase):
    def to_dict(self):
        dictionary = self.__dict__.copy()
        return dict((k, v) for k, v in dictionary.items()
                    if k != '_sa_instance_state')


Base = declarative.declarative_base(cls=_MuranoBase)


class Environment(Base, TimestampMixin):
    """Represents an Environment in the metadata-store."""
    __tablename__ = 'environment'
    __table_args__ = (sa.Index(
        'ix_name_tenant_id', 'name', 'tenant_id', unique=True),)

    id = sa.Column(sa.String(255),
                   primary_key=True,
                   default=uuidutils.generate_uuid)
    name = sa.Column(sa.String(255), nullable=False)
    tenant_id = sa.Column(sa.String(36), nullable=False)
    description_text = sa.Column(sa.String(), nullable=False, default='')
    version = sa.Column(sa.BigInteger, nullable=False, default=0)
    description = sa.Column(st.JsonBlob(), nullable=False, default={})

    sessions = sa_orm.relationship("Session", backref='environment',
                                   cascade='save-update, merge, delete')

    tasks = sa_orm.relationship('Task', backref='environment',
                                cascade='save-update, merge, delete')

    cf_spaces = sa_orm.relationship("CFSpace", backref='environment',
                                    cascade='save-update, merge, delete')

    cf_serv_inst = sa_orm.relationship("CFServiceInstance",
                                       backref='environment',
                                       cascade='save-update, merge, delete')

    def to_dict(self):
        dictionary = super(Environment, self).to_dict()
        del dictionary['description']
        return dictionary


class EnvironmentTemplate(Base, TimestampMixin):
    """Represents an Environment Template in the metadata-store."""
    __tablename__ = 'environment-template'

    id = sa.Column(sa.String(36),
                   primary_key=True,
                   default=uuidutils.generate_uuid)
    name = sa.Column(sa.String(255), nullable=False)
    tenant_id = sa.Column(sa.String(36), nullable=False)
    description_text = sa.Column(sa.String(), nullable=False, default='')
    version = sa.Column(sa.BigInteger, nullable=False, default=0)
    description = sa.Column(st.JsonBlob(), nullable=False, default={})
    is_public = sa.Column(sa.Boolean, default=False)

    def to_dict(self):
        dictionary = super(EnvironmentTemplate, self).to_dict()
        if 'description' in dictionary:
            del dictionary['description']
        return dictionary


class Session(Base, TimestampMixin):
    __tablename__ = 'session'

    id = sa.Column(sa.String(36),
                   primary_key=True,
                   default=uuidutils.generate_uuid)
    environment_id = sa.Column(sa.String(255), sa.ForeignKey('environment.id'))

    user_id = sa.Column(sa.String(64), nullable=False)
    state = sa.Column(sa.String(36), nullable=False)
    description = sa.Column(st.JsonBlob(), nullable=False)
    version = sa.Column(sa.BigInteger, nullable=False, default=0)

    def to_dict(self):
        dictionary = super(Session, self).to_dict()
        del dictionary['description']
        # object relations may be not loaded yet
        if 'environment' in dictionary:
            del dictionary['environment']
        return dictionary


class Task(Base, TimestampMixin):
    __tablename__ = 'task'

    id = sa.Column(sa.String(36), primary_key=True,
                   default=uuidutils.generate_uuid)
    started = sa.Column(sa.DateTime, default=timeutils.utcnow, nullable=False)
    finished = sa.Column(sa.DateTime, default=None, nullable=True)
    description = sa.Column(st.JsonBlob(), nullable=False)
    environment_id = sa.Column(sa.String(255), sa.ForeignKey('environment.id'))
    action = sa.Column(st.JsonBlob())

    statuses = sa_orm.relationship("Status", backref='task',
                                   cascade='save-update, merge, delete')
    result = sa.Column(st.JsonBlob(), nullable=True, default={})

    def to_dict(self):
        dictionary = super(Task, self).to_dict()
        if 'statuses' in dictionary:
            del dictionary['statuses']
        if 'environment' in dictionary:
            del dictionary['environment']
        return dictionary


class Status(Base, TimestampMixin):
    __tablename__ = 'status'

    id = sa.Column(sa.String(36),
                   primary_key=True,
                   default=uuidutils.generate_uuid)
    entity_id = sa.Column(sa.String(255), nullable=True)
    entity = sa.Column(sa.String(10), nullable=True)
    task_id = sa.Column(sa.String(32), sa.ForeignKey('task.id'))
    text = sa.Column(sa.Text(), nullable=False)
    level = sa.Column(sa.String(32), nullable=False)
    details = sa.Column(sa.Text(), nullable=True)

    def to_dict(self):
        dictionary = super(Status, self).to_dict()
        # object relations may be not loaded yet
        if 'deployment' in dictionary:
            del dictionary['deployment']
        return dictionary


class ApiStats(Base, TimestampMixin):
    __tablename__ = 'apistats'

    id = sa.Column(sa.Integer(), primary_key=True)
    host = sa.Column(sa.String(80))
    request_count = sa.Column(sa.BigInteger())
    error_count = sa.Column(sa.BigInteger())
    average_response_time = sa.Column(sa.Float())
    requests_per_tenant = sa.Column(sa.Text())
    requests_per_second = sa.Column(sa.Float())
    errors_per_second = sa.Column(sa.Float())
    cpu_count = sa.Column(sa.Integer())
    cpu_percent = sa.Column(sa.Float())


package_to_category = sa.Table('package_to_category',
                               Base.metadata,
                               sa.Column('package_id',
                                         sa.String(36),
                                         sa.ForeignKey('package.id')),
                               sa.Column('category_id',
                                         sa.String(36),
                                         sa.ForeignKey('category.id',
                                                       ondelete="RESTRICT")))

package_to_tag = sa.Table('package_to_tag',
                          Base.metadata,
                          sa.Column('package_id',
                                    sa.String(36),
                                    sa.ForeignKey('package.id')),
                          sa.Column('tag_id',
                                    sa.String(36),
                                    sa.ForeignKey('tag.id',
                                                  ondelete="CASCADE")))


class Instance(Base):
    __tablename__ = 'instance_stats'

    environment_id = sa.Column(
        sa.String(255), primary_key=True, nullable=False)
    instance_id = sa.Column(
        sa.String(255), primary_key=True, nullable=False)
    instance_type = sa.Column(sa.Integer, default=0, nullable=False)
    created = sa.Column(sa.Integer, nullable=False)
    destroyed = sa.Column(sa.Integer, nullable=True)
    type_name = sa.Column('type_name', sa.String(512), nullable=False)
    type_title = sa.Column('type_title', sa.String(512))
    unit_count = sa.Column('unit_count', sa.Integer())
    tenant_id = sa.Column('tenant_id', sa.String(36), nullable=False)


class Package(Base, TimestampMixin):
    """Represents a meta information about application package."""
    __tablename__ = 'package'
    __table_args__ = (sa.Index('ix_package_fqn_and_owner',
                               'fully_qualified_name',
                               'owner_id', unique=True),)
    id = sa.Column(sa.String(36),
                   primary_key=True,
                   default=uuidutils.generate_uuid)
    archive = sa.Column(st.LargeBinary())
    fully_qualified_name = sa.Column(sa.String(128),
                                     nullable=False)
    type = sa.Column(sa.String(20), nullable=False, default='class')
    author = sa.Column(sa.String(80), default='OpenStack')
    supplier = sa.Column(st.JsonBlob(), nullable=True, default={})
    name = sa.Column(sa.String(80), nullable=False)
    enabled = sa.Column(sa.Boolean, default=True)
    description = sa.Column(sa.Text(),
                            nullable=False,
                            default='The description is not provided')
    is_public = sa.Column(sa.Boolean, default=False)
    tags = sa_orm.relationship("Tag",
                               secondary=package_to_tag,
                               cascade='save-update, merge',
                               lazy='joined')
    logo = sa.Column(st.LargeBinary(), nullable=True)
    owner_id = sa.Column(sa.String(64), nullable=False)
    ui_definition = sa.Column(sa.Text)
    supplier_logo = sa.Column(sa.LargeBinary, nullable=True)
    categories = sa_orm.relationship("Category",
                                     secondary=package_to_category,
                                     cascade='save-update, merge',
                                     lazy='joined')
    class_definitions = sa_orm.relationship(
        "Class", cascade='save-update, merge, delete', lazy='joined',
        backref='package')

    def to_dict(self):
        d = self.__dict__.copy()
        not_serializable = ['_sa_instance_state',
                            'archive',
                            'logo',
                            'ui_definition',
                            'supplier_logo']
        nested_objects = ['categories', 'tags', 'class_definitions']
        for key in not_serializable:
            if key in d.keys():
                del d[key]
        for key in nested_objects:
            d[key] = [a.name for a in d.get(key, [])]
        return d


class Category(Base, TimestampMixin):
    """Represents an application categories in the datastore."""
    __tablename__ = 'category'

    id = sa.Column(sa.String(36),
                   primary_key=True,
                   default=uuidutils.generate_uuid)
    name = sa.Column(sa.String(80), nullable=False, index=True, unique=True)

    package_count = sa_orm.column_property(
        sa.select([sa.func.count(package_to_category.c.package_id)]).
        where(package_to_category.c.category_id == id).
        correlate_except(package_to_category)
    )

    def to_dict(self):
        d = super(Category, self).to_dict()
        d['package_count'] = self.package_count
        return d


class Tag(Base, TimestampMixin):
    """Represents tags in the datastore."""
    __tablename__ = 'tag'

    id = sa.Column(sa.String(36),
                   primary_key=True,
                   default=uuidutils.generate_uuid)
    name = sa.Column(sa.String(80), nullable=False, unique=True)


class Class(Base, TimestampMixin):
    """Represents a class definition in the datastore."""
    __tablename__ = 'class_definition'

    id = sa.Column(sa.String(36),
                   primary_key=True,
                   default=uuidutils.generate_uuid)
    name = sa.Column(sa.String(128), nullable=False, index=True)
    package_id = sa.Column(sa.String(36), sa.ForeignKey('package.id'))


class Lock(Base):
    __tablename__ = 'locks'
    id = sa.Column(sa.String(50), primary_key=True)
    ts = sa.Column(sa.DateTime, nullable=False)


class CFOrganization(Base):
    __tablename__ = "cf_orgs"
    id = sa.Column(sa.String(255), primary_key=True)
    tenant = sa.Column(sa.String(255), nullable=False)


class CFSpace(Base):
    __tablename__ = "cf_spaces"
    id = sa.Column(sa.String(255), primary_key=True)
    environment_id = sa.Column(sa.String(255), sa.ForeignKey('environment.id'),
                               nullable=False)

    def to_dict(self):
        dictionary = super(CFSpace, self).to_dict()
        if 'environment' in dictionary:
            del dictionary['environment']
        return dictionary


class CFServiceInstance(Base):
    __tablename__ = 'cf_serv_inst'
    id = sa.Column(sa.String(255), primary_key=True)
    service_id = sa.Column(sa.String(255), nullable=False)
    environment_id = sa.Column(sa.String(255), sa.ForeignKey('environment.id'),
                               nullable=False)
    tenant = sa.Column(sa.String(255), nullable=False)

    def to_dict(self):
        dictionary = super(CFServiceInstance, self).to_dict()
        if 'environment' in dictionary:
            del dictionary['environment']
        return dictionary


def register_models(engine):
    """Creates database tables for all models with the given engine."""
    models = (Environment, Status, Session, Task,
              ApiStats, Package, Category, Class, Instance, Lock, CFSpace,
              CFOrganization)
    for model in models:
        model.metadata.create_all(engine)


def unregister_models(engine):
    """Drops database tables for all models with the given engine."""
    models = (Environment, Status, Session, Task,
              ApiStats, Package, Category, Class, Lock, CFOrganization,
              CFSpace)
    for model in models:
        model.metadata.drop_all(engine)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Create the environment-template table for the environment
template functionality.

Revision ID: 005
Revises: table template

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '005'
down_revision = '004'

MYSQL_ENGINE = 'InnoDB'
MYSQL_CHARSET = 'utf8'


def upgrade():
    """It creates the table environment-template.

    The name plus the tenant_id should be unique in the table,
    since each tenant cannot duplicate template names.
    """

    op.create_table(
        'environment-template',
        sa.Column('created', sa.DateTime(), nullable=False),
        sa.Column('updated', sa.DateTime(), nullable=False),
        sa.Column('id', sa.String(length=255), nullable=False),
        sa.Column('name', sa.String(length=255), nullable=False),
        sa.Column('tenant_id', sa.String(length=36), nullable=False),
        sa.Column('version', sa.BigInteger(), nullable=False),
        sa.Column('description', sa.Text(), nullable=False),
        sa.Column('networking', sa.Text(), nullable=True),
        sa.PrimaryKeyConstraint('id'),
        sa.UniqueConstraint('tenant_id', 'name'),
        mysql_engine=MYSQL_ENGINE,
        mysql_charset=MYSQL_CHARSET
    )


def downgrade():
    op.drop_table('environment-template')
